@@ -41,7 +41,7 @@ let options =
     Arg.(value & opt_all string [] & info [ "I" ] ~docv:"DIR" ~doc)
   in
   Term.(
-    pure (fun files output_file include_dirs -> { files; output_file; include_dirs })
+    const (fun files output_file include_dirs -> { files; output_file; include_dirs })
     $ files
     $ output_file
     $ include_dirs)
@@ -61,20 +61,27 @@ function jsoo_create_file_extern(name,content){
 }
 |}
   in
-  let fragments = Linker.parse_string code in
+  let fragments = Linker.Fragment.parse_string code in
   Linker.load_fragments ~target_env:Isomorphic ~filename:"<dummy>" fragments;
+  Linker.check_deps ();
+  let include_dirs =
+    List.filter_map (include_dirs @ [ "+stdlib/" ]) ~f:(fun d -> Findlib.find [] d)
+  in
   let instr =
     Pseudo_fs.f ~prim:`create_file_extern ~cmis:StringSet.empty ~files ~paths:include_dirs
   in
   let code = Code.prepend Code.empty instr in
   Filename.gen_file output_file (fun chan ->
       let pfs_fmt = Pretty_print.to_out_channel chan in
-      Driver.f
-        ~standalone:true
-        ~wrap_with_fun:`Iife
-        pfs_fmt
-        (Parse_bytecode.Debug.create ~toplevel:false false)
-        code)
+      let (_ : Source_map.t option) =
+        Driver.f
+          ~standalone:true
+          ~wrap_with_fun:`Iife
+          pfs_fmt
+          (Parse_bytecode.Debug.create ~include_cmis:false false)
+          code
+      in
+      ())
 
 let info =
   Info.make
@@ -83,4 +90,6 @@ let info =
     ~description:
       "jsoo_fs is a tool for embeding files in a Js_of_ocaml pseudo filesystem."
 
-let command = Cmdliner.Term.(pure f $ options), info
+let command =
+  let t = Cmdliner.Term.(const f $ options) in
+  Cmdliner.Cmd.v info t
