@@ -54,9 +54,10 @@ let f (runtime_files, bytecode, target_env) =
   in
   List.iter builtin ~f:(fun t ->
       let filename = Builtins.File.name t in
-      let runtimes = Linker.parse_builtin t in
+      let runtimes = Linker.Fragment.parse_builtin t in
       Linker.load_fragments ~target_env ~filename runtimes);
   Linker.load_files ~target_env runtime_files;
+  Linker.check_deps ();
   let all_prims =
     List.concat_map bytecode ~f:(fun f ->
         let ic = open_in_bin f in
@@ -148,7 +149,7 @@ let options =
     Arg.(
       value & opt (enum options) Target_env.Isomorphic & info [ "target-env" ] ~docv ~doc)
   in
-  let t = Term.(pure build_t $ files $ target_env) in
+  let t = Term.(const build_t $ files $ target_env) in
   Term.ret t
 
 let info =
@@ -157,4 +158,6 @@ let info =
     ~doc:"Check runtime"
     ~description:"js_of_ocaml-check-runtime checks the runtime."
 
-let command = Cmdliner.Term.(pure f $ options), info
+let command =
+  let t = Cmdliner.Term.(const f $ options) in
+  Cmdliner.Cmd.v info t
