@@ -21,6 +21,24 @@ open! Js_of_ocaml_compiler.Stdlib
 open Js_of_ocaml_compiler
 open Cmdliner
 
+let is_dir_sep = function
+  | '/' -> true
+  | '\\' when String.equal Filename.dir_sep "\\" -> true
+  | _ -> false
+
+let trim_trailing_dir_sep s =
+  if String.equal s ""
+  then s
+  else
+    let len = String.length s in
+    let j = ref (len - 1) in
+    while !j >= 0 && is_dir_sep (String.unsafe_get s !j) do
+      decr j
+    done;
+    if !j >= 0 then String.sub s ~pos:0 ~len:(!j + 1) else String.sub s ~pos:0 ~len:1
+
+let normalize_include_dirs dirs = List.map dirs ~f:trim_trailing_dir_sep
+
 type t =
   { common : Jsoo_cmdline.Arg.t
   ; (* compile option *)
@@ -40,9 +58,9 @@ type t =
   ; linkall : bool
   ; toplevel : bool
   ; export_file : string option
-  ; nocmis : bool
+  ; no_cmis : bool
   ; (* filesystem *)
-    include_dir : string list
+    include_dirs : string list
   ; fs_files : string list
   ; fs_output : string option
   ; fs_external : bool
@@ -68,8 +86,9 @@ let wrap_with_fun_conv =
   in
   Arg.conv (conv, printer)
 
+let toplevel_section = "OPTIONS (TOPLEVEL)"
+
 let options =
-  let toplevel_section = "OPTIONS (TOPLEVEL)" in
   let filesystem_section = "OPTIONS (FILESYSTEM)" in
   let js_files =
     let doc =
@@ -162,26 +181,40 @@ let options =
       value & opt (enum options) Target_env.Isomorphic & info [ "target-env" ] ~docv ~doc)
   in
   let toplevel =
-    let doc = "Compile a toplevel." in
+    let doc =
+      "Compile a toplevel and embed necessary cmis (unless '--no-cmis' is provided). \
+       Exported compilation units can be configured with '--export'.  Note you you'll \
+       also need to link against js_of_ocaml-toplevel."
+    in
     Arg.(value & flag & info [ "toplevel" ] ~docs:toplevel_section ~doc)
   in
   let export_file =
-    let doc = "File containing the list of unit to export in a toplevel." in
+    let doc =
+      "File containing the list of unit to export in a toplevel, with Dynlink or with \
+       --linkall. If absent, all units will be exported."
+    in
     Arg.(value & opt (some string) None & info [ "export" ] ~docs:toplevel_section ~doc)
   in
-  let linkall =
-    let doc = "Link all primitives." in
-    Arg.(value & flag & info [ "linkall" ] ~doc)
-  in
   let dynlink =
-    let doc = "Enable dynlink." in
+    let doc =
+      "Enable dynlink of bytecode files.  Use this if you want to be able to use the \
+       Dynlink module. Note that you'll also need to link with \
+       'js_of_ocaml-compiler.dynlink'."
+    in
     Arg.(value & flag & info [ "dynlink" ] ~doc)
   in
-  let nocmis =
+  let linkall =
+    let doc =
+      "Link all primitives and compilation units. Exported compilation units can be \
+       configured with '--export'."
+    in
+    Arg.(value & flag & info [ "linkall" ] ~doc)
+  in
+  let no_cmis =
     let doc = "Do not include cmis when compiling toplevel." in
     Arg.(value & flag & info [ "nocmis"; "no-cmis" ] ~docs:toplevel_section ~doc)
   in
-  let include_dir =
+  let include_dirs =
     let doc = "Add [$(docv)] to the list of include directories." in
     Arg.(
       value & opt_all string [] & info [ "I" ] ~docs:filesystem_section ~docv:"DIR" ~doc)
@@ -230,11 +263,11 @@ let options =
       toplevel
       export_file
       wrap_with_fun
-      include_dir
+      include_dirs
       fs_files
       fs_output
       fs_external
-      nocmis
+      no_cmis
       profile
       no_runtime
       runtime_only
@@ -255,8 +288,7 @@ let options =
       then runtime_files @ [ input_file ]
       else runtime_files
     in
-    let linkall = linkall || toplevel || runtime_only in
-    let fs_external = fs_external || (toplevel && nocmis) || runtime_only in
+    let fs_external = fs_external || (toplevel && no_cmis) || runtime_only in
     let input_file =
       match input_file, runtime_only with
       | "-", _ | _, true -> None
@@ -304,6 +336,7 @@ let options =
     in
     let params : (string * string) list = List.flatten set_param in
     let static_env : (string * string) list = List.flatten set_env in
+    let include_dirs = normalize_include_dirs include_dirs in
     `Ok
       { common
       ; params
@@ -315,14 +348,14 @@ let options =
       ; target_env
       ; toplevel
       ; export_file
-      ; include_dir
+      ; include_dirs
       ; runtime_files
       ; no_runtime
       ; runtime_only
       ; fs_files
       ; fs_output
       ; fs_external
-      ; nocmis
+      ; no_cmis
       ; output_file
       ; input_file
       ; source_map
@@ -331,8 +364,8 @@ let options =
   in
   let t =
     Term.(
-      pure build_t
-      $ Jsoo_cmdline.Arg.t
+      const build_t
+      $ Lazy.force Jsoo_cmdline.Arg.t
       $ set_param
       $ set_env
       $ dynlink
@@ -340,11 +373,11 @@ let options =
       $ toplevel
       $ export_file
       $ wrap_with_function
-      $ include_dir
+      $ include_dirs
       $ fs_files
       $ fs_output
       $ fs_external
-      $ nocmis
+      $ no_cmis
       $ profile
       $ noruntime
       $ runtime_only
@@ -401,6 +434,18 @@ let options_runtime_only =
     let doc = "root dir for source map." in
     Arg.(value & opt (some string) None & info [ "source-map-root" ] ~doc)
   in
+  let toplevel =
+    let doc =
+      "Compile a toplevel and embed necessary cmis (unless '--no-cmis' is provided). \
+       Exported compilation units can be configured with '--export'.  Note you you'll \
+       also need to link against js_of_ocaml-toplevel."
+    in
+    Arg.(value & flag & info [ "toplevel" ] ~docs:toplevel_section ~doc)
+  in
+  let no_cmis =
+    let doc = "Do not include cmis when compiling toplevel." in
+    Arg.(value & flag & info [ "nocmis"; "no-cmis" ] ~docs:toplevel_section ~doc)
+  in
   let target_env =
     let doc = "Runtime compile target." in
     let options = List.map ~f:(fun env -> Target_env.to_string env, env) Target_env.all in
@@ -430,7 +475,7 @@ let options_runtime_only =
       & opt_all (list (pair ~sep:'=' string string)) []
       & info [ "setenv" ] ~docv:"PARAM=VALUE" ~doc)
   in
-  let include_dir =
+  let include_dirs =
     let doc = "Add [$(docv)] to the list of include directories." in
     Arg.(
       value & opt_all string [] & info [ "I" ] ~docs:filesystem_section ~docv:"DIR" ~doc)
@@ -472,13 +517,15 @@ let options_runtime_only =
   in
   let build_t
       common
+      toplevel
+      no_cmis
       set_param
       set_env
       wrap_with_fun
       fs_files
       fs_output
       fs_external
-      include_dir
+      include_dirs
       no_runtime
       no_sourcemap
       sourcemap
@@ -528,6 +575,7 @@ let options_runtime_only =
     in
     let params : (string * string) list = List.flatten set_param in
     let static_env : (string * string) list = List.flatten set_env in
+    let include_dirs = normalize_include_dirs include_dirs in
     `Ok
       { common
       ; params
@@ -537,16 +585,16 @@ let options_runtime_only =
       ; dynlink = false
       ; linkall = false
       ; target_env
-      ; toplevel = false
+      ; toplevel
       ; export_file = None
-      ; include_dir
+      ; include_dirs
       ; runtime_files
       ; no_runtime
       ; runtime_only = true
       ; fs_files
       ; fs_output
       ; fs_external
-      ; nocmis = true
+      ; no_cmis
       ; output_file
       ; input_file = None
       ; source_map
@@ -555,15 +603,17 @@ let options_runtime_only =
   in
   let t =
     Term.(
-      pure build_t
-      $ Jsoo_cmdline.Arg.t
+      const build_t
+      $ Lazy.force Jsoo_cmdline.Arg.t
+      $ toplevel
+      $ no_cmis
       $ set_param
       $ set_env
       $ wrap_with_function
       $ fs_files
       $ fs_output
       $ fs_external
-      $ include_dir
+      $ include_dirs
       $ noruntime
       $ no_sourcemap
       $ sourcemap
