@@ -53,7 +53,7 @@ end = struct
 end
 
 module Var : sig
-  type t
+  type t [@@ocaml.immediate]
 
   val print : Format.formatter -> t -> unit
 
@@ -107,6 +107,8 @@ module Var : sig
     val set : 'a t -> key -> 'a -> unit
 
     val make : size -> 'a -> 'a t
+
+    val iter : (key -> 'a -> unit) -> 'a t -> unit
   end
 
   module ISet : sig
@@ -196,8 +198,6 @@ end = struct
     propagate_name o n;
     n
 
-  let dummy = -1
-
   module Set = Set.Make (T)
   module Map = Map.Make (T)
 
@@ -213,28 +213,29 @@ end = struct
     let set t x v = t.(x) <- v
 
     let make () v = Array.make (count ()) v
-  end
-
-  module ISet = struct
-    type t = T.t array
-
-    type elt = T.t
 
     let iter f t =
       for i = 0 to Array.length t - 1 do
-        let x = t.(i) in
-        if compare x dummy <> 0 then f x
+        f i (Array.unsafe_get t i)
       done
+  end
 
-    let mem t x = compare t.(x) dummy <> 0
+  module ISet = struct
+    type t = BitSet.t
 
-    let add t x = t.(x) <- x
+    type elt = T.t
 
-    let remove t x = t.(x) <- dummy
+    let iter f t = BitSet.iter ~f t
 
-    let copy = Array.copy
+    let mem t x = BitSet.mem t x
 
-    let empty _v = Array.make (count ()) dummy
+    let add t (x : int) = BitSet.set t x
+
+    let remove t x = BitSet.unset t x
+
+    let copy = BitSet.copy
+
+    let empty _v = BitSet.create' (count ())
   end
 end
 
@@ -257,9 +258,25 @@ type array_or_not =
   | NotArray
   | Unknown
 
+module Native_string = struct
+  type t =
+    | Byte of string
+    | Utf of Utf8_string.t
+
+  let of_string x = Utf (Utf8_string.of_string_exn x)
+
+  let of_bytestring x = if String.is_ascii x then of_string x else Byte x
+
+  let equal a b =
+    match a, b with
+    | Byte x, Byte y -> String.equal x y
+    | Utf (Utf8 x), Utf (Utf8 y) -> String.equal x y
+    | Utf _, Byte _ | Byte _, Utf _ -> false
+end
+
 type constant =
   | String of string
-  | NativeString of string
+  | NativeString of Native_string.t
   | Float of float
   | Float_array of float array
   | Int64 of int64
@@ -269,7 +286,7 @@ type constant =
 let rec constant_equal a b =
   match a, b with
   | String a, String b -> Some (String.equal a b)
-  | NativeString a, NativeString b -> Some (String.equal a b)
+  | NativeString a, NativeString b -> Some (Native_string.equal a b)
   | Tuple (ta, a, _), Tuple (tb, b, _) ->
       if ta <> tb || Array.length a <> Array.length b
       then Some false
@@ -303,12 +320,21 @@ let rec constant_equal a b =
   | Int _, (String _ | NativeString _ | Float_array _ | Int64 _ | Tuple (_, _, _)) ->
       Some false
 
+type loc =
+  | No
+  | Before of Addr.t
+  | After of Addr.t
+
 type prim_arg =
   | Pv of Var.t
   | Pc of constant
 
 type expr =
-  | Apply of Var.t * Var.t list * bool
+  | Apply of
+      { f : Var.t
+      ; args : Var.t list
+      ; exact : bool
+      }
   | Block of int * Var.t array * array_or_not
   | Field of Var.t * int
   | Closure of Var.t list * cont
@@ -317,6 +343,7 @@ type expr =
 
 type instr =
   | Let of Var.t * expr
+  | Assign of Var.t * Var.t
   | Set_field of Var.t * int * Var.t
   | Offset_ref of Var.t * int
   | Array_set of Var.t * Var.t * Var.t
@@ -327,15 +354,14 @@ type last =
   | Stop
   | Branch of cont
   | Cond of Var.t * cont * cont
-  | Switch of Var.t * cont array * cont array
+  | Switch of Var.t * cont array
   | Pushtrap of cont * Var.t * cont * Addr.Set.t
-  | Poptrap of cont * Addr.t
+  | Poptrap of cont
 
 type block =
   { params : Var.t list
-  ; handler : (Var.t * cont) option
-  ; body : instr list
-  ; branch : last
+  ; body : (instr * loc) list
+  ; branch : last * loc
   }
 
 type program =
@@ -344,6 +370,9 @@ type program =
   ; free_pc : Addr.t
   }
 
+let noloc = No
+
+let location_of_pc pc = Before pc
 (****)
 
 module Print = struct
@@ -360,7 +389,8 @@ module Print = struct
   let rec constant f x =
     match x with
     | String s -> Format.fprintf f "%S" s
-    | NativeString s -> Format.fprintf f "%Sj" s
+    | NativeString (Byte s) -> Format.fprintf f "%Sj" s
+    | NativeString (Utf (Utf8 s)) -> Format.fprintf f "%Sj" s
     | Float fl -> Format.fprintf f "%.12g" fl
     | Float_array a ->
         Format.fprintf f "[|";
@@ -435,10 +465,10 @@ module Print = struct
 
   let expr f e =
     match e with
-    | Apply (g, l, exact) ->
+    | Apply { f = g; args; exact } ->
         if exact
-        then Format.fprintf f "%a!(%a)" Var.print g var_list l
-        else Format.fprintf f "%a(%a)" Var.print g var_list l
+        then Format.fprintf f "%a!(%a)" Var.print g var_list args
+        else Format.fprintf f "%a(%a)" Var.print g var_list args
     | Block (t, a, _) ->
         Format.fprintf f "{tag=%d" t;
         for i = 0 to Array.length a - 1 do
@@ -450,15 +480,16 @@ module Print = struct
     | Constant c -> Format.fprintf f "CONST{%a}" constant c
     | Prim (p, l) -> prim f p l
 
-  let instr f i =
+  let instr f (i, _loc) =
     match i with
     | Let (x, e) -> Format.fprintf f "%a = %a" Var.print x expr e
+    | Assign (x, y) -> Format.fprintf f "(assign) %a = %a" Var.print x Var.print y
     | Set_field (x, i, y) -> Format.fprintf f "%a[%d] = %a" Var.print x i Var.print y
     | Offset_ref (x, i) -> Format.fprintf f "%a[0] += %d" Var.print x i
     | Array_set (x, y, z) ->
         Format.fprintf f "%a[%a] = %a" Var.print x Var.print y Var.print z
 
-  let last f l =
+  let last f (l, _loc) =
     match l with
     | Return x -> Format.fprintf f "return %a" Var.print x
     | Raise (x, `Normal) -> Format.fprintf f "raise %a" Var.print x
@@ -468,10 +499,9 @@ module Print = struct
     | Branch c -> Format.fprintf f "branch %a" cont c
     | Cond (x, cont1, cont2) ->
         Format.fprintf f "if %a then %a else %a" Var.print x cont cont1 cont cont2
-    | Switch (x, a1, a2) ->
+    | Switch (x, a1) ->
         Format.fprintf f "switch %a {" Var.print x;
         Array.iteri a1 ~f:(fun i c -> Format.fprintf f "int %d -> %a; " i cont c);
-        Array.iteri a2 ~f:(fun i c -> Format.fprintf f "tag %d -> %a; " i cont c);
         Format.fprintf f "}"
     | Pushtrap (cont1, x, cont2, pcs) ->
         Format.fprintf
@@ -484,17 +514,14 @@ module Print = struct
           cont
           cont2
           (String.concat ~sep:", " (List.map (Addr.Set.elements pcs) ~f:string_of_int))
-    | Poptrap (c, _) -> Format.fprintf f "poptrap %a" cont c
+    | Poptrap c -> Format.fprintf f "poptrap %a" cont c
 
   type xinstr =
-    | Instr of instr
-    | Last of last
+    | Instr of (instr * loc)
+    | Last of (last * loc)
 
   let block annot pc block =
     Format.eprintf "==== %d (%a) ====@." pc var_list block.params;
-    (match block.handler with
-    | Some (x, c) -> Format.eprintf "    handler %a => %a@." Var.print x cont c
-    | None -> ());
     List.iter block.body ~f:(fun i ->
         Format.eprintf " %s %a@." (annot pc (Instr i)) instr i);
     Format.eprintf " %s %a@." (annot pc (Last block.branch)) last block.branch;
@@ -510,7 +537,7 @@ end
 let fold_closures p f accu =
   Addr.Map.fold
     (fun _ block accu ->
-      List.fold_left block.body ~init:accu ~f:(fun accu i ->
+      List.fold_left block.body ~init:accu ~f:(fun accu (i, _loc) ->
           match i with
           | Let (x, Closure (params, cont)) -> f (Some x) params cont accu
           | _ -> accu))
@@ -522,16 +549,21 @@ let fold_closures p f accu =
 let prepend ({ start; blocks; free_pc } as p) body =
   match body with
   | [] -> p
-  | _ ->
-      let new_start = free_pc in
-      let branch = if Addr.Map.mem start blocks then Branch (start, []) else Stop in
-      let blocks =
-        Addr.Map.add new_start { params = []; handler = None; body; branch } blocks
-      in
-      let free_pc = free_pc + 1 in
-      { start = new_start; blocks; free_pc }
+  | _ -> (
+      match Addr.Map.find start blocks with
+      | block ->
+          { p with
+            blocks = Addr.Map.add start { block with body = body @ block.body } blocks
+          }
+      | exception Not_found ->
+          let new_start = free_pc in
+          let blocks =
+            Addr.Map.add new_start { params = []; body; branch = Stop, noloc } blocks
+          in
+          let free_pc = free_pc + 1 in
+          { start = new_start; blocks; free_pc })
 
-let empty_block = { params = []; handler = None; body = []; branch = Stop }
+let empty_block = { params = []; body = []; branch = Stop, noloc }
 
 let empty =
   let start = 0 in
@@ -544,30 +576,29 @@ let is_empty p =
   | 1 -> (
       let _, v = Addr.Map.choose p.blocks in
       match v with
-      | { handler = None; body; branch = Stop; params = _ } -> (
+      | { body; branch = Stop, _; params = _ } -> (
           match body with
-          | ([] | [ Let (_, Prim (Extern "caml_get_global_data", _)) ]) when true -> true
+          | ([] | [ (Let (_, Prim (Extern "caml_get_global_data", _)), _) ]) when true ->
+              true
           | _ -> false)
       | _ -> false)
   | _ -> false
 
 let fold_children blocks pc f accu =
   let block = Addr.Map.find pc blocks in
-  let accu =
-    match block.handler with
-    | Some (_, (pc, _)) -> f pc accu
-    | None -> accu
-  in
-  match block.branch with
+  match fst block.branch with
   | Return _ | Raise _ | Stop -> accu
-  | Branch (pc', _) | Poptrap ((pc', _), _) | Pushtrap ((pc', _), _, _, _) -> f pc' accu
+  | Branch (pc', _) | Poptrap (pc', _) -> f pc' accu
+  | Pushtrap ((pc', _), _, (pc_h, _), _) ->
+      let accu = f pc' accu in
+      let accu = f pc_h accu in
+      accu
   | Cond (_, (pc1, _), (pc2, _)) ->
       let accu = f pc1 accu in
       let accu = f pc2 accu in
       accu
-  | Switch (_, a1, a2) ->
+  | Switch (_, a1) ->
       let accu = Array.fold_right ~init:accu ~f:(fun (pc, _) accu -> f pc accu) a1 in
-      let accu = Array.fold_right ~init:accu ~f:(fun (pc, _) accu -> f pc accu) a2 in
       accu
 
 type 'c fold_blocs = block Addr.Map.t -> Addr.t -> (Addr.t -> 'c -> 'c) -> 'c -> 'c
@@ -593,6 +624,42 @@ let rec traverse' { fold } f pc visited blocks acc =
 
 let traverse fold f pc blocks acc = snd (traverse' fold f pc Addr.Set.empty blocks acc)
 
+let rec preorder_traverse' { fold } f pc visited blocks acc =
+  if not (Addr.Set.mem pc visited)
+  then
+    let visited = Addr.Set.add pc visited in
+    let acc = f pc acc in
+    fold
+      blocks
+      pc
+      (fun pc (visited, acc) ->
+        let visited, acc = preorder_traverse' { fold } f pc visited blocks acc in
+        visited, acc)
+      (visited, acc)
+  else visited, acc
+
+let preorder_traverse fold f pc blocks acc =
+  snd (preorder_traverse' fold f pc Addr.Set.empty blocks acc)
+
+let fold_closures_innermost_first { start; blocks; _ } f accu =
+  let rec visit blocks pc f accu =
+    traverse
+      { fold = fold_children }
+      (fun pc accu ->
+        let block = Addr.Map.find pc blocks in
+        List.fold_left block.body ~init:accu ~f:(fun accu i ->
+            match i with
+            | Let (x, Closure (params, cont)), _ ->
+                let accu = visit blocks (fst cont) f accu in
+                f (Some x) params cont accu
+            | _ -> accu))
+      pc
+      blocks
+      accu
+  in
+  let accu = visit blocks start f accu in
+  f None [] (start, []) accu
+
 let eq p1 p2 =
   p1.start = p2.start
   && Addr.Map.cardinal p1.blocks = Addr.Map.cardinal p2.blocks
@@ -617,19 +684,19 @@ let invariant { blocks; start; _ } =
   if with_invariant ()
   then (
     assert (Addr.Map.mem start blocks);
-    let defs = Array.make (Var.count ()) false in
+    let defs = Var.ISet.empty () in
     let check_cont (cont, args) =
       let b = Addr.Map.find cont blocks in
-      assert (List.length args >= List.length b.params)
+      assert (List.length args = List.length b.params)
     in
     let define x =
       if check_defs
       then (
-        assert (not defs.(Var.idx x));
-        defs.(Var.idx x) <- true)
+        assert (not (Var.ISet.mem defs x));
+        Var.ISet.add defs x)
     in
     let check_expr = function
-      | Apply (_, _, _) -> ()
+      | Apply _ -> ()
       | Block (_, _, _) -> ()
       | Field (_, _) -> ()
       | Closure (l, cont) ->
@@ -638,15 +705,18 @@ let invariant { blocks; start; _ } =
       | Constant _ -> ()
       | Prim (_, _) -> ()
     in
-    let check_instr = function
+    let check_instr (i, _loc) =
+      match i with
       | Let (x, e) ->
           define x;
           check_expr e
+      | Assign _ -> ()
       | Set_field (_, _i, _) -> ()
       | Offset_ref (_x, _i) -> ()
       | Array_set (_x, _y, _z) -> ()
     in
-    let check_last = function
+    let check_last (l, _loc) =
+      match l with
       | Return _ -> ()
       | Raise _ -> ()
       | Stop -> ()
@@ -654,18 +724,15 @@ let invariant { blocks; start; _ } =
       | Cond (_x, cont1, cont2) ->
           check_cont cont1;
           check_cont cont2
-      | Switch (_x, a1, a2) ->
-          Array.iteri a1 ~f:(fun _ cont -> check_cont cont);
-          Array.iteri a2 ~f:(fun _ cont -> check_cont cont)
+      | Switch (_x, a1) -> Array.iteri a1 ~f:(fun _ cont -> check_cont cont)
       | Pushtrap (cont1, _x, cont2, _pcs) ->
           check_cont cont1;
           check_cont cont2
-      | Poptrap (cont, _) -> check_cont cont
+      | Poptrap cont -> check_cont cont
     in
     Addr.Map.iter
       (fun _pc block ->
         List.iter block.params ~f:define;
-        Option.iter block.handler ~f:(fun (_, cont) -> check_cont cont);
         List.iter block.body ~f:check_instr;
         check_last block.branch)
       blocks)
