@@ -46,7 +46,7 @@ module DebugAddr : sig
 end
 
 module Var : sig
-  type t
+  type t [@@ocaml.immediate]
 
   val print : Format.formatter -> t -> unit
 
@@ -100,6 +100,8 @@ module Var : sig
     val set : 'a t -> key -> 'a -> unit
 
     val make : size -> 'a -> 'a t
+
+    val iter : (key -> 'a -> unit) -> 'a t -> unit
   end
 
   module ISet : sig
@@ -140,9 +142,19 @@ type array_or_not =
   | NotArray
   | Unknown
 
+module Native_string : sig
+  type t = private
+    | Byte of string
+    | Utf of Utf8_string.t
+
+  val of_string : string -> t
+
+  val of_bytestring : string -> t
+end
+
 type constant =
   | String of string
-  | NativeString of string
+  | NativeString of Native_string.t
   | Float of float
   | Float_array of float array
   | Int64 of int64
@@ -151,13 +163,25 @@ type constant =
 
 val constant_equal : constant -> constant -> bool option
 
+type loc =
+  | No
+  | Before of Addr.t
+  | After of Addr.t
+
+val noloc : loc
+
+val location_of_pc : int -> loc
+
 type prim_arg =
   | Pv of Var.t
   | Pc of constant
 
 type expr =
-  | Apply of Var.t * Var.t list * bool
-  (* if true, then # of arguments = # of parameters *)
+  | Apply of
+      { f : Var.t
+      ; args : Var.t list
+      ; exact : bool (* if true, then # of arguments = # of parameters *)
+      }
   | Block of int * Var.t array * array_or_not
   | Field of Var.t * int
   | Closure of Var.t list * cont
@@ -166,6 +190,7 @@ type expr =
 
 type instr =
   | Let of Var.t * expr
+  | Assign of Var.t * Var.t
   | Set_field of Var.t * int * Var.t
   | Offset_ref of Var.t * int
   | Array_set of Var.t * Var.t * Var.t
@@ -176,15 +201,14 @@ type last =
   | Stop
   | Branch of cont
   | Cond of Var.t * cont * cont
-  | Switch of Var.t * cont array * cont array
+  | Switch of Var.t * cont array
   | Pushtrap of cont * Var.t * cont * Addr.Set.t
-  | Poptrap of cont * Addr.t
+  | Poptrap of cont
 
 type block =
   { params : Var.t list
-  ; handler : (Var.t * cont) option
-  ; body : instr list
-  ; branch : last
+  ; body : (instr * loc) list
+  ; branch : last * loc
   }
 
 type program =
@@ -195,18 +219,22 @@ type program =
 
 module Print : sig
   type xinstr =
-    | Instr of instr
-    | Last of last
+    | Instr of (instr * loc)
+    | Last of (last * loc)
+
+  val expr : Format.formatter -> expr -> unit
+
+  val constant : Format.formatter -> constant -> unit
 
   val var_list : Format.formatter -> Var.t list -> unit
 
-  val instr : Format.formatter -> instr -> unit
+  val instr : Format.formatter -> instr * loc -> unit
 
   val block : (Addr.Map.key -> xinstr -> string) -> int -> block -> unit
 
   val program : (Addr.Map.key -> xinstr -> string) -> program -> unit
 
-  val last : Format.formatter -> last -> unit
+  val last : Format.formatter -> last * loc -> unit
 
   val cont : Format.formatter -> cont -> unit
 end
@@ -217,13 +245,30 @@ type fold_blocs_poly = { fold : 'a. 'a fold_blocs } [@@unboxed]
 
 val fold_closures :
   program -> (Var.t option -> Var.t list -> cont -> 'd -> 'd) -> 'd -> 'd
+(** [fold_closures p f init] folds [f] over all closures in the program [p],
+    starting from the initial value [init]. For each closure, [f] is called
+    with the following arguments: the closure name (enclosed in
+    {!Stdlib.Some}), its parameter list, the address and parameter instantiation
+    of its first block, and the current accumulator. In addition, [f] is called
+    on the initial block [p.start], with [None] as the closure name.
+    All closures in all blocks of [p] are included in the fold, not only the
+    ones reachable from [p.start]. *)
+
+val fold_closures_innermost_first :
+  program -> (Var.t option -> Var.t list -> cont -> 'd -> 'd) -> 'd -> 'd
+(** Similar to {!fold_closures}, but applies the fold function to the
+    innermost closures first. Unlike with {!fold_closures}, only the closures
+    reachable from [p.start] are considered. *)
 
 val fold_children : 'c fold_blocs
 
 val traverse :
   fold_blocs_poly -> (Addr.t -> 'c -> 'c) -> Addr.t -> block Addr.Map.t -> 'c -> 'c
 
-val prepend : program -> instr list -> program
+val preorder_traverse :
+  fold_blocs_poly -> (Addr.t -> 'c -> 'c) -> Addr.t -> block Addr.Map.t -> 'c -> 'c
+
+val prepend : program -> (instr * loc) list -> program
 
 val empty : program
 
