@@ -23,6 +23,11 @@ let debug = Debug.find "main"
 
 let times = Debug.find "times"
 
+type profile =
+  | O1
+  | O2
+  | O3
+
 let should_export = function
   | `Iife -> false
   | `Named _ | `Anonymous -> true
@@ -49,7 +54,7 @@ let inline p =
 
 let specialize_1 (p, info) =
   if debug () then Format.eprintf "Specialize...@.";
-  Specialize.f info p
+  Specialize.f ~function_arity:(fun f -> Specialize.function_arity info f) p
 
 let specialize_js (p, info) =
   if debug () then Format.eprintf "Specialize js...@.";
@@ -80,12 +85,50 @@ let phi p =
   if debug () then Format.eprintf "Variable passing simplification...@.";
   Phisimpl.f p
 
+let ( +> ) f g x = g (f x)
+
+let map_fst f (x, y) = f x, y
+
+let effects ~deadcode_sentinal p =
+  if Config.Flag.effects ()
+  then (
+    if debug () then Format.eprintf "Effects...@.";
+    let p, live_vars = Deadcode.f p in
+    let p = Effects.remove_empty_blocks ~live_vars p in
+    let p, live_vars = Deadcode.f p in
+    let info = Global_flow.f ~fast:false p in
+    let p, live_vars =
+      if Config.Flag.globaldeadcode ()
+      then
+        let p = Global_deadcode.f p ~deadcode_sentinal info in
+        Deadcode.f p
+      else p, live_vars
+    in
+    let p, cps = p |> Effects.f ~flow_info:info ~live_vars +> map_fst Lambda_lifting.f in
+    p, cps)
+  else p, (Code.Var.Set.empty : Effects.cps_calls)
+
+let exact_calls profile ~deadcode_sentinal p =
+  if not (Config.Flag.effects ())
+  then
+    let fast =
+      match profile with
+      | O3 -> false
+      | O1 | O2 -> true
+    in
+    let info = Global_flow.f ~fast p in
+    let p =
+      if Config.Flag.globaldeadcode () && Config.Flag.deadcode ()
+      then Global_deadcode.f p ~deadcode_sentinal info
+      else p
+    in
+    Specialize.f ~function_arity:(fun f -> Global_flow.function_arity info f) p
+  else p
+
 let print p =
   if debug () then Code.Print.program (fun _ _ -> "") p;
   p
 
-let ( +> ) f g x = g (f x)
-
 let rec loop max name round i (p : 'a) : 'a =
   let p' = round p in
   if i >= max || Code.eq p' p
@@ -144,33 +187,36 @@ let round2 = flow +> specialize' +> eval +> deadcode +> o1
 
 let o3 = loop 10 "tailcall+inline" round1 1 +> loop 10 "flow" round2 1 +> print
 
-let generate d ~exported_runtime ~wrap_with_fun (p, live_vars) =
+let generate
+    d
+    ~exported_runtime
+    ~wrap_with_fun
+    ~warn_on_unhandled_effect
+    ~deadcode_sentinal
+    ((p, live_vars), cps_calls) =
   if times () then Format.eprintf "Start Generation...@.";
   let should_export = should_export wrap_with_fun in
-  Generate.f p ~exported_runtime ~live_vars ~should_export d
-
-let header formatter ~custom_header =
-  (match custom_header with
-  | None -> ()
-  | Some c -> Pretty_print.string formatter (c ^ "\n"));
-  let version =
-    match Compiler_version.git_version with
-    | "" -> Compiler_version.s
-    | v -> Printf.sprintf "%s+git-%s" Compiler_version.s v
-  in
-  Pretty_print.string formatter ("// Generated by js_of_ocaml " ^ version ^ "\n")
+  Generate.f
+    p
+    ~exported_runtime
+    ~live_vars
+    ~cps_calls
+    ~should_export
+    ~warn_on_unhandled_effect
+    ~deadcode_sentinal
+    d
 
 let debug_linker = Debug.find "linker"
 
-let global_object = Constant.global_object
-
 let extra_js_files =
   lazy
     (List.fold_left (Builtins.all ()) ~init:[] ~f:(fun acc file ->
          try
            let name = Builtins.File.name file in
            let ss =
-             List.concat_map ~f:Linker.Fragment.provides (Linker.parse_builtin file)
+             List.concat_map
+               ~f:Linker.Fragment.provides
+               (Linker.Fragment.parse_builtin file)
              |> StringSet.of_list
            in
            (name, ss) :: acc
@@ -197,36 +243,37 @@ let report_missing_primitives missing =
 
 let gen_missing js missing =
   let open Javascript in
+  let ident_s n = ident (Utf8_string.of_string_exn n) in
   let miss =
     StringSet.fold
       (fun prim acc ->
+        let prim = Utf8_string.of_string_exn prim in
         let p = ident prim in
         ( p
-        , Some
-            ( ECond
-                ( EBin
-                    ( NotEqEq
-                    , EDot (EVar (ident global_object), prim)
-                    , EVar (ident "undefined") )
-                , EDot (EVar (ident global_object), prim)
-                , EFun
-                    ( None
-                    , []
-                    , [ ( Statement
-                            (Expression_statement
-                               (ECall
-                                  ( EVar (ident "caml_failwith")
-                                  , [ ( EBin
-                                          ( Plus
-                                          , EStr (prim, `Utf8)
-                                          , EStr (" not implemented", `Utf8) )
-                                      , `Not_spread )
-                                    ]
-                                  , N )))
+        , ( ECond
+              ( EBin
+                  ( NotEqEq
+                  , dot (EVar (ident Constant.global_object_)) prim
+                  , EVar (ident_s "undefined") )
+              , dot (EVar (ident Constant.global_object_)) prim
+              , EFun
+                  ( None
+                  , fun_
+                      []
+                      [ ( Expression_statement
+                            (call
+                               (EVar (ident_s "caml_failwith"))
+                               [ EBin
+                                   ( Plus
+                                   , EStr prim
+                                   , EStr (Utf8_string.of_string_exn " not implemented")
+                                   )
+                               ]
+                               N)
                         , N )
                       ]
-                    , N ) )
-            , N ) )
+                      N ) )
+          , N ) )
         :: acc)
       missing
       []
@@ -239,10 +286,11 @@ let gen_missing js missing =
     warn "You can prevent the generation of dummy implementations with ";
     warn "the commandline option '--disable genprim'@.";
     report_missing_primitives missing);
-  (Statement (Variable_statement miss), N) :: js
+  (variable_declaration miss, N) :: js
 
-let link ~standalone ~linkall ~export_runtime (js : Javascript.source_elements) :
-    Linker.output =
+let mark_start_of_generated_code = Debug.find ~even_if_quiet:true "mark-runtime-gen"
+
+let link ~standalone ~linkall (js : Javascript.statement_list) : Linker.output =
   if not standalone
   then { runtime_code = js; always_required_codes = [] }
   else
@@ -250,7 +298,31 @@ let link ~standalone ~linkall ~export_runtime (js : Javascript.source_elements)
     if times () then Format.eprintf "Start Linking...@.";
     let traverse = new Js_traverse.free in
     let js = traverse#program js in
-    let free = traverse#get_free_name in
+    let js =
+      if mark_start_of_generated_code ()
+      then
+        let open Javascript in
+        ( Expression_statement
+            (EStr
+               (Utf8_string.of_string_exn ("--MARK--" ^ "start-of-jsoo-gen" ^ "--MARK--")))
+        , N )
+        :: js
+      else js
+    in
+    let free = traverse#get_free in
+    let free : StringSet.t =
+      Javascript.IdentSet.fold
+        (fun x acc ->
+          match x with
+          | V _ ->
+              (* This is an error. We don't complain here as we want
+                 to be able to name other variable to make it
+                 easier to spot the problematic ones *)
+              acc
+          | S { name = Utf8 x; _ } -> StringSet.add x acc)
+        free
+        StringSet.empty
+    in
     let prim = Primitive.get_external () in
     let prov = Linker.get_provided () in
     let all_external = StringSet.union prim prov in
@@ -270,14 +342,22 @@ let link ~standalone ~linkall ~export_runtime (js : Javascript.source_elements)
     let js = if Config.Flag.genprim () then gen_missing js missing else js in
     if times () then Format.eprintf "  linking: %a@." Timer.print t;
     let js =
-      if export_runtime
+      if linkall
       then
         let open Javascript in
         let all = Linker.all linkinfos in
-        let all = List.map all ~f:(fun name -> PNI name, EVar (ident name)) in
-        ( Statement
-            (Expression_statement
-               (EBin (Eq, EDot (EVar (ident global_object), "jsoo_runtime"), EObj all)))
+        let all =
+          List.map all ~f:(fun name ->
+              let name = Utf8_string.of_string_exn name in
+              Property (PNI name, EVar (ident name)))
+        in
+        ( Expression_statement
+            (EBin
+               ( Eq
+               , dot
+                   (EVar (ident Constant.global_object_))
+                   (Utf8_string.of_string_exn "jsoo_runtime")
+               , EObj all ))
         , N )
         :: js
       else js
@@ -289,7 +369,16 @@ let check_js js =
   if times () then Format.eprintf "Start Checks...@.";
   let traverse = new Js_traverse.free in
   let js = traverse#program js in
-  let free = traverse#get_free_name in
+  let free = traverse#get_free in
+  let free : StringSet.t =
+    Javascript.IdentSet.fold
+      (fun x acc ->
+        match x with
+        | V _ -> assert false
+        | S { name = Utf8 x; _ } -> StringSet.add x acc)
+      free
+      StringSet.empty
+  in
   let prim = Primitive.get_external () in
   let prov = Linker.get_provided () in
   let all_external = StringSet.union prim prov in
@@ -317,18 +406,27 @@ let coloring js =
   if times () then Format.eprintf "Start Coloring...@.";
   let traverse = new Js_traverse.free in
   let js = traverse#program js in
-  let free = traverse#get_free_name in
-  Var_printer.add_reserved (StringSet.elements free);
+  let free = traverse#get_free in
+  Javascript.IdentSet.iter
+    (fun x ->
+      match x with
+      | V _ ->
+          (* This is an error. We don't complain here as we want
+             to be able to name other variable to make it
+             easier to spot the problematic ones *)
+          ()
+      | S { name = Utf8 x; _ } -> Var_printer.add_reserved x)
+    free;
   let js = Js_assign.program js in
   if times () then Format.eprintf "  coloring: %a@." Timer.print t;
   js
 
-let output formatter ~standalone ~custom_header ~source_map () js =
+let output formatter ~source_map () js =
   let t = Timer.make () in
   if times () then Format.eprintf "Start Writing file...@.";
-  if standalone then header ~custom_header formatter;
-  Js_output.program formatter ?source_map js;
-  if times () then Format.eprintf "  write: %a@." Timer.print t
+  let sm = Js_output.program formatter ?source_map js in
+  if times () then Format.eprintf "  write: %a@." Timer.print t;
+  sm
 
 let pack ~wrap_with_fun ~standalone { Linker.runtime_code = js; always_required_codes } =
   let module J = Javascript in
@@ -355,52 +453,56 @@ let pack ~wrap_with_fun ~standalone { Linker.runtime_code = js; always_required_
   in
   (* pack *)
   let wrap_in_iife ~use_strict js =
-    let var ident e =
-      J.Statement (J.Variable_statement [ J.ident ident, Some (e, J.N) ]), J.N
-    in
-    let expr e = J.Statement (J.Expression_statement e), J.N in
+    let var ident e = J.variable_declaration [ J.ident ident, (e, J.N) ], J.N in
+    let expr e = J.Expression_statement e, J.N in
     let freenames =
       let o = new Js_traverse.free in
       let (_ : J.program) = o#program js in
-      o#get_free_name
+      o#get_free
     in
     let export_shim js =
-      if StringSet.mem Constant.exports freenames
+      if J.IdentSet.mem (J.ident Constant.exports_) freenames
       then
         if should_export wrap_with_fun
-        then var Constant.exports (J.EObj []) :: js
+        then var Constant.exports_ (J.EObj []) :: js
         else
           let export_node =
             let s =
               Printf.sprintf
                 {|((typeof module === 'object' && module.exports) || %s)|}
-                global_object
+                Constant.global_object
             in
-            let lex = Parse_js.Lexer.of_lexbuf (Lexing.from_string s) in
+            let lex = Parse_js.Lexer.of_string s in
             Parse_js.parse_expr lex
           in
-          var Constant.exports export_node :: js
+          var Constant.exports_ export_node :: js
       else js
     in
     let old_global_object_shim js =
-      if StringSet.mem Constant.old_global_object freenames
-      then var Constant.old_global_object (J.EVar (J.ident global_object)) :: js
+      if J.IdentSet.mem (J.ident Constant.old_global_object_) freenames
+      then
+        var Constant.old_global_object_ (J.EVar (J.ident Constant.global_object_)) :: js
       else js
     in
 
-    let efun args body = J.EFun (None, args, body, J.U) in
-    let sfun name args body = J.Function_declaration (name, args, body, J.U), J.U in
+    let efun args body = J.EFun (None, J.fun_ args body J.U) in
+    let sfun name args body = J.Function_declaration (name, J.fun_ args body J.U), J.U in
     let mk f =
       let js = export_shim js in
       let js = old_global_object_shim js in
-      let js = if use_strict then expr (J.EStr ("use strict", `Utf8)) :: js else js in
-      f [ J.ident global_object ] js
+      let js =
+        if use_strict
+        then expr (J.EStr (Utf8_string.of_string_exn "use strict")) :: js
+        else js
+      in
+      f [ J.ident Constant.global_object_ ] js
     in
     match wrap_with_fun with
     | `Anonymous -> expr (mk efun)
-    | `Named name -> mk (sfun (J.ident name))
-    | `Iife ->
-        expr (J.ECall (mk efun, [ J.EVar (J.ident global_object), `Not_spread ], J.N))
+    | `Named name ->
+        let name = Utf8_string.of_string_exn name in
+        mk (sfun (J.ident name))
+    | `Iife -> expr (J.call (mk efun) [ J.EVar (J.ident Constant.global_object_) ] J.N)
   in
   let always_required_js =
     (* consider adding a comments in the generated file with original
@@ -419,6 +521,7 @@ let pack ~wrap_with_fun ~standalone { Linker.runtime_code = js; always_required_
   let js =
     match wrap_with_fun, standalone with
     | `Named name, (true | false) ->
+        assert (J.is_ident name);
         let export_node =
           let s =
             Printf.sprintf
@@ -429,7 +532,7 @@ if (typeof module === 'object' && module.exports) {
 |}
               name
           in
-          let lex = Parse_js.Lexer.of_lexbuf (Lexing.from_string s) in
+          let lex = Parse_js.Lexer.of_string s in
           Parse_js.parse lex
         in
         js @ export_node
@@ -456,7 +559,7 @@ if (typeof module === 'object' && module.exports) {
 }(Object));
 |}
           in
-          let lex = Parse_js.Lexer.of_lexbuf (Lexing.from_string s) in
+          let lex = Parse_js.Lexer.of_string s in
           Parse_js.parse lex
         in
         e @ js
@@ -472,8 +575,7 @@ if (typeof module === 'object' && module.exports) {
     if Config.Flag.shortvar ()
     then (
       let t5 = Timer.make () in
-      let keep = StringSet.empty in
-      let js = (new Js_traverse.rename_variable keep)#program js in
+      let js = (new Js_traverse.rename_variable ~esm:false)#program js in
       if times () then Format.eprintf "    shortten vars: %a@." Timer.print t5;
       js)
     else js
@@ -481,43 +583,40 @@ if (typeof module === 'object' && module.exports) {
   if times () then Format.eprintf "  optimizing: %a@." Timer.print t;
   js
 
-let configure formatter p =
+let configure formatter =
   let pretty = Config.Flag.pretty () in
   Pretty_print.set_compact formatter (not pretty);
   Code.Var.set_pretty (pretty && not (Config.Flag.shortvar ()));
-  Code.Var.set_stable (Config.Flag.stable_var ());
-  p
+  Code.Var.set_stable (Config.Flag.stable_var ())
 
-type profile = Code.program -> Code.program
-
-let full
-    ~standalone
-    ~wrap_with_fun
-    ~profile
-    ~dynlink
-    ~linkall
-    ~source_map
-    ~custom_header
-    formatter
-    d
-    p =
+let full ~standalone ~wrap_with_fun ~profile ~linkall ~source_map formatter d p =
   let exported_runtime = not standalone in
-  let linkall = linkall || dynlink in
-
+  let deadcode_sentinal =
+    (* If deadcode is disabled, this field is just fresh variable *)
+    Code.Var.fresh_n "undef"
+  in
   let opt =
-    configure formatter
-    +> specialize_js_once
-    +> profile
-    +> Generate_closure.f
-    +> deadcode'
+    specialize_js_once
+    +> (match profile with
+       | O1 -> o1
+       | O2 -> o2
+       | O3 -> o3)
+    +> exact_calls ~deadcode_sentinal profile
+    +> effects ~deadcode_sentinal
+    +> map_fst (Generate_closure.f +> deadcode')
   in
   let emit =
-    generate d ~exported_runtime ~wrap_with_fun
-    +> link ~standalone ~linkall ~export_runtime:dynlink
+    generate
+      d
+      ~exported_runtime
+      ~wrap_with_fun
+      ~warn_on_unhandled_effect:standalone
+      ~deadcode_sentinal
+    +> link ~standalone ~linkall
     +> pack ~wrap_with_fun ~standalone
     +> coloring
     +> check_js
-    +> output formatter ~standalone ~custom_header ~source_map ()
+    +> output formatter ~source_map ()
   in
   if times () then Format.eprintf "Start Optimizing...@.";
   let t = Timer.make () in
@@ -525,43 +624,44 @@ let full
   let () = if times () then Format.eprintf " optimizations : %a@." Timer.print t in
   emit r
 
+let full_no_source_map ~standalone ~wrap_with_fun ~profile ~linkall formatter d p =
+  let (_ : Source_map.t option) =
+    full ~standalone ~wrap_with_fun ~profile ~linkall ~source_map:None formatter d p
+  in
+  ()
+
 let f
     ?(standalone = true)
     ?(wrap_with_fun = `Iife)
-    ?(profile = o1)
-    ?(dynlink = false)
+    ?(profile = O1)
     ?(linkall = false)
     ?source_map
-    ?custom_header
     formatter
     d
     p =
-  full
-    ~standalone
-    ~wrap_with_fun
-    ~profile
-    ~dynlink
-    ~linkall
-    ~source_map
-    ~custom_header
+  full ~standalone ~wrap_with_fun ~profile ~linkall ~source_map formatter d p
+
+let f'
+    ?(standalone = true)
+    ?(wrap_with_fun = `Iife)
+    ?(profile = O1)
+    ?(linkall = false)
     formatter
     d
-    p
+    p =
+  full_no_source_map ~standalone ~wrap_with_fun ~profile ~linkall formatter d p
 
-let from_string prims s formatter =
-  let p, d = Parse_bytecode.from_string prims s in
-  full
+let from_string ~prims ~debug s formatter =
+  let p, d = Parse_bytecode.from_string ~prims ~debug s in
+  full_no_source_map
     ~standalone:false
     ~wrap_with_fun:`Anonymous
-    ~profile:o1
-    ~dynlink:false
+    ~profile:O1
     ~linkall:false
-    ~source_map:None
-    ~custom_header:None
     formatter
     d
     p
 
-let profiles = [ 1, o1; 2, o2; 3, o3 ]
+let profiles = [ 1, O1; 2, O2; 3, O3 ]
 
 let profile i = try Some (List.assoc i profiles) with Not_found -> None
