@@ -24,16 +24,31 @@ val f :
      ?standalone:bool
   -> ?wrap_with_fun:[ `Iife | `Anonymous | `Named of string ]
   -> ?profile:profile
-  -> ?dynlink:bool
   -> ?linkall:bool
-  -> ?source_map:string option * Source_map.t
-  -> ?custom_header:string
+  -> ?source_map:Source_map.t
+  -> Pretty_print.t
+  -> Parse_bytecode.Debug.t
+  -> Code.program
+  -> Source_map.t option
+
+val f' :
+     ?standalone:bool
+  -> ?wrap_with_fun:[ `Iife | `Anonymous | `Named of string ]
+  -> ?profile:profile
+  -> ?linkall:bool
   -> Pretty_print.t
   -> Parse_bytecode.Debug.t
   -> Code.program
   -> unit
 
-val from_string : string array -> string -> Pretty_print.t -> unit
+val from_string :
+     prims:string array
+  -> debug:Instruct.debug_event list array
+  -> string
+  -> Pretty_print.t
+  -> unit
+
+val configure : Pretty_print.t -> unit
 
 val profiles : (int * profile) list
 
