@@ -27,7 +27,9 @@ type closure_info =
   ; args : Code.Var.t list
   ; cont : Code.cont
   ; tc : Code.Addr.Set.t Code.Var.Map.t
-  ; ntc : Code.Addr.Set.t Code.Var.Map.t
+  ; mutated_vars : Code.Var.Set.t
+  ; loc : Code.loc
+  ; pos : int
   }
 
 type 'a int_ext =
@@ -41,48 +43,38 @@ let add_multi k v map =
   let set = try Var.Map.find k map with Not_found -> Addr.Set.empty in
   Var.Map.add k (Addr.Set.add v set) map
 
-let rec collect_apply pc blocks visited tc ntc =
+let rec collect_apply pc blocks visited tc =
   if Addr.Set.mem pc visited
-  then visited, tc, ntc
+  then visited, tc
   else
     let visited = Addr.Set.add pc visited in
     let block = Addr.Map.find pc blocks in
     let tc_opt =
-      match block.branch with
+      match fst block.branch with
       | Return x -> (
           match List.last block.body with
-          | Some (Let (y, Apply (z, _, true))) when Code.Var.compare x y = 0 ->
-              Some (add_multi z pc tc)
+          | Some (Let (y, Apply { f; exact = true; _ }), _) when Code.Var.compare x y = 0
+            -> Some (add_multi f pc tc)
           | None -> None
           | Some _ -> None)
       | _ -> None
     in
-    let visited, ntc =
-      List.fold_left block.body ~init:(visited, ntc) ~f:(fun (visited, acc) x ->
-          match x with
-          | Let (_, Apply (z, _, _)) -> visited, add_multi z pc acc
-          | Let (_, Closure (_, (pc, _))) ->
-              let visited, _tc, ntc = collect_apply pc blocks visited tc ntc in
-              visited, ntc
-          | _ -> visited, acc)
-    in
     match tc_opt with
-    | Some tc -> visited, tc, ntc
+    | Some tc -> visited, tc
     | None ->
         Code.fold_children
           blocks
           pc
-          (fun pc (visited, tc, ntc) -> collect_apply pc blocks visited tc ntc)
-          (visited, tc, ntc)
+          (fun pc (visited, tc) -> collect_apply pc blocks visited tc)
+          (visited, tc)
 
-let rec collect_closures blocks l =
+let rec collect_closures blocks mutated_vars l pos =
   match l with
-  | Let (f_name, Closure (args, ((pc, _) as cont))) :: rem ->
-      let _, tc, ntc =
-        collect_apply pc blocks Addr.Set.empty Var.Map.empty Var.Map.empty
-      in
-      let l, rem = collect_closures blocks rem in
-      { f_name; args; cont; tc; ntc } :: l, rem
+  | (Let (f_name, Closure (args, ((pc, _) as cont))), loc) :: rem ->
+      let _, tc = collect_apply pc blocks Addr.Set.empty Var.Map.empty in
+      let l, rem = collect_closures blocks mutated_vars rem (succ pos) in
+      let mutated_vars = Addr.Map.find pc mutated_vars in
+      { f_name; args; cont; tc; mutated_vars; loc; pos } :: l, rem
   | rem -> [], rem
 
 let group_closures ~tc_only closures_map =
@@ -93,14 +85,7 @@ let group_closures ~tc_only closures_map =
     Var.Map.fold
       (fun _ x graph ->
         let calls = Var.Map.fold (fun x _ tc -> Var.Set.add x tc) x.tc Var.Set.empty in
-        let calls =
-          if tc_only
-          then calls
-          else
-            Var.Set.union
-              calls
-              (Var.Map.fold (fun x _ ntc -> Var.Set.add x ntc) x.ntc Var.Set.empty)
-        in
+        let calls = if tc_only then calls else Var.Set.union calls x.mutated_vars in
         Var.Map.add x.f_name (Var.Set.inter names calls) graph)
       closures_map
       Var.Map.empty
@@ -109,45 +94,57 @@ let group_closures ~tc_only closures_map =
   SCC.connected_components_sorted_from_roots_to_leaf graph
 
 module Trampoline = struct
-  let direct_call_block block ~counter ~x ~f ~args =
-    let counter_plus_1 = Code.Var.fork counter in
+  let direct_call_block ~counter ~x ~f ~args loc =
     let return = Code.Var.fork x in
-    { block with
-      params = []
-    ; body =
-        [ Let (counter_plus_1, Prim (Extern "%int_add", [ Pv counter; Pc (Int 1l) ]))
-        ; Let (return, Apply (f, counter_plus_1 :: args, true))
-        ]
-    ; branch = Return return
-    }
+    match counter with
+    | None ->
+        { params = []
+        ; body = [ Let (return, Apply { f; args; exact = true }), loc ]
+        ; branch = Return return, loc
+        }
+    | Some counter ->
+        let counter_plus_1 = Code.Var.fork counter in
+        { params = []
+        ; body =
+            [ ( Let (counter_plus_1, Prim (Extern "%int_add", [ Pv counter; Pc (Int 1l) ]))
+              , noloc )
+            ; Let (return, Apply { f; args = counter_plus_1 :: args; exact = true }), loc
+            ]
+        ; branch = Return return, loc
+        }
 
-  let bounce_call_block block ~x ~f ~args =
+  let bounce_call_block ~x ~f ~args loc =
     let return = Code.Var.fork x in
     let new_args = Code.Var.fresh () in
-    { block with
-      params = []
+    { params = []
     ; body =
-        [ Let
-            ( new_args
-            , Prim (Extern "%js_array", Pc (Int 0l) :: List.map args ~f:(fun x -> Pv x))
-            )
-        ; Let (return, Prim (Extern "caml_trampoline_return", [ Pv f; Pv new_args ]))
+        [ ( Let
+              ( new_args
+              , Prim (Extern "%js_array", Pc (Int 0l) :: List.map args ~f:(fun x -> Pv x))
+              )
+          , noloc )
+        ; Let (return, Prim (Extern "caml_trampoline_return", [ Pv f; Pv new_args ])), loc
         ]
-    ; branch = Return return
+    ; branch = Return return, loc
     }
 
-  let wrapper_block f ~args ~counter =
+  let wrapper_block f ~args ~counter loc =
     let result1 = Code.Var.fresh () in
     let result2 = Code.Var.fresh () in
     let block =
       { params = []
-      ; handler = None
       ; body =
-          [ Let (counter, Constant (Int 0l))
-          ; Let (result1, Apply (f, counter :: args, true))
-          ; Let (result2, Prim (Extern "caml_trampoline", [ Pv result1 ]))
-          ]
-      ; branch = Return result2
+          (match counter with
+          | None ->
+              [ Let (result1, Apply { f; args; exact = true }), loc
+              ; Let (result2, Prim (Extern "caml_trampoline", [ Pv result1 ])), noloc
+              ]
+          | Some counter ->
+              [ Let (counter, Constant (Int 0l)), noloc
+              ; Let (result1, Apply { f; args = counter :: args; exact = true }), loc
+              ; Let (result2, Prim (Extern "caml_trampoline", [ Pv result1 ])), noloc
+              ])
+      ; branch = Return result2, loc
       }
     in
     block
@@ -158,7 +155,7 @@ module Trampoline = struct
     match component with
     | SCC.No_loop id ->
         let ci = Var.Map.find id closures_map in
-        let instr = Let (ci.f_name, Closure (ci.args, ci.cont)) in
+        let instr = Let (ci.f_name, Closure (ci.args, ci.cont)), ci.loc in
         free_pc, blocks, { int = []; ext = [ instr ] }
     | SCC.Has_loop all ->
         if debug_tc ()
@@ -167,9 +164,11 @@ module Trampoline = struct
           Format.eprintf
             "%s\n%!"
             (String.concat ~sep:", " (List.map all ~f:(fun x -> Var.to_string x))));
+        let tailcall_max_depth = Config.Param.tailcall_max_depth () in
         let all =
           List.map all ~f:(fun id ->
-              Code.Var.fresh_n "counter", Var.Map.find id closures_map)
+              ( (if tailcall_max_depth = 0 then None else Some (Code.Var.fresh_n "counter"))
+              , Var.Map.find id closures_map ))
         in
         let blocks, free_pc, instrs, instrs_wrapper =
           List.fold_left
@@ -182,13 +181,20 @@ module Trampoline = struct
               let new_args = List.map ci.args ~f:Code.Var.fork in
               let wrapper_pc = free_pc in
               let free_pc = free_pc + 1 in
-              let new_counter = Code.Var.fork counter in
+              let new_counter = Option.map counter ~f:Code.Var.fork in
               let wrapper_block =
-                wrapper_block new_f ~args:new_args ~counter:new_counter
+                wrapper_block new_f ~args:new_args ~counter:new_counter ci.loc
               in
               let blocks = Addr.Map.add wrapper_pc wrapper_block blocks in
-              let instr_wrapper = Let (ci.f_name, wrapper_closure wrapper_pc new_args) in
-              let instr_real = Let (new_f, Closure (counter :: ci.args, ci.cont)) in
+              let instr_wrapper =
+                Let (ci.f_name, wrapper_closure wrapper_pc new_args), ci.loc
+              in
+              let instr_real =
+                match counter with
+                | None -> Let (new_f, Closure (ci.args, ci.cont)), ci.loc
+                | Some counter ->
+                    Let (new_f, Closure (counter :: ci.args, ci.cont)), ci.loc
+              in
               let counter_and_pc =
                 List.fold_left all ~init:[] ~f:(fun acc (counter, ci2) ->
                     try
@@ -207,38 +213,42 @@ module Trampoline = struct
                     let bounce_call_pc = free_pc + 1 in
                     let free_pc = free_pc + 2 in
                     match List.rev block.body with
-                    | Let (x, Apply (f, args, true)) :: rem_rev ->
+                    | (Let (x, Apply { f; args; exact = true }), loc) :: rem_rev ->
                         assert (Var.equal f ci.f_name);
                         let blocks =
                           Addr.Map.add
                             direct_call_pc
-                            (direct_call_block block ~counter ~x ~f:new_f ~args)
+                            (direct_call_block ~counter ~x ~f:new_f ~args loc)
                             blocks
                         in
                         let blocks =
                           Addr.Map.add
                             bounce_call_pc
-                            (bounce_call_block block ~x ~f:new_f ~args)
+                            (bounce_call_block ~x ~f:new_f ~args loc)
                             blocks
                         in
-                        let direct = Code.Var.fresh () in
-                        let branch =
-                          Cond (direct, (direct_call_pc, []), (bounce_call_pc, []))
-                        in
-                        let last =
-                          Let
-                            ( direct
-                            , Prim
-                                ( Lt
-                                , [ Pv counter
-                                  ; Pc
-                                      (Int
-                                         (Int32.of_int
-                                            (Config.Param.tailcall_max_depth ())))
-                                  ] ) )
-                        in
                         let block =
-                          { block with body = List.rev (last :: rem_rev); branch }
+                          match counter with
+                          | None ->
+                              let branch = Branch (bounce_call_pc, []), loc in
+                              { block with body = List.rev rem_rev; branch }
+                          | Some counter ->
+                              let direct = Code.Var.fresh () in
+                              let branch =
+                                ( Cond (direct, (direct_call_pc, []), (bounce_call_pc, []))
+                                , loc )
+                              in
+                              let last =
+                                ( Let
+                                    ( direct
+                                    , Prim
+                                        ( Lt
+                                        , [ Pv counter
+                                          ; Pc (Int (Int32.of_int tailcall_max_depth))
+                                          ] ) )
+                                , noloc )
+                              in
+                              { block with body = List.rev (last :: rem_rev); branch }
                         in
                         let blocks = Addr.Map.remove pc blocks in
                         Addr.Map.add pc block blocks, free_pc
@@ -246,7 +256,7 @@ module Trampoline = struct
               in
               blocks, free_pc, instr_real :: instrs, instr_wrapper :: instrs_wrapper)
         in
-        free_pc, blocks, { int = instrs; ext = instrs_wrapper }
+        free_pc, blocks, { int = List.rev instrs; ext = List.rev instrs_wrapper }
 end
 
 module Ident = struct
@@ -254,13 +264,13 @@ module Ident = struct
     match component with
     | SCC.No_loop id ->
         let ci = Var.Map.find id closures_map in
-        let instr = Let (ci.f_name, Closure (ci.args, ci.cont)) in
+        let instr = Let (ci.f_name, Closure (ci.args, ci.cont)), ci.loc in
         free_pc, blocks, { int = []; ext = [ instr ] }
     | SCC.Has_loop ids ->
         let instrs =
           List.map ids ~f:(fun id ->
               let ci = Var.Map.find id closures_map in
-              let instr = Let (ci.f_name, Closure (ci.args, ci.cont)) in
+              let instr = Let (ci.f_name, Closure (ci.args, ci.cont)), ci.loc in
               instr)
         in
         free_pc, blocks, { int = []; ext = instrs }
@@ -268,9 +278,16 @@ end
 
 let rewrite_tc free_pc blocks closures_map component =
   let open Config.Param in
-  match tailcall_optim () with
-  | TcNone -> Ident.f free_pc blocks closures_map component
-  | TcTrampoline -> Trampoline.f free_pc blocks closures_map component
+  let trampoline =
+    (not (Config.Flag.effects ()))
+    &&
+    match tailcall_optim () with
+    | TcTrampoline -> true
+    | TcNone -> false
+  in
+  if trampoline
+  then Trampoline.f free_pc blocks closures_map component
+  else Ident.f free_pc blocks closures_map component
 
 let rewrite_mutable
     free_pc
@@ -286,7 +303,7 @@ let rewrite_mutable
       ~init:(Var.Set.empty, Var.Set.empty)
       ~f:(fun (all_mut, names) i ->
         match i with
-        | Let (x, Closure (_, (pc, _))) ->
+        | Let (x, Closure (_, (pc, _))), _ ->
             let all_mut =
               try Var.Set.union all_mut (Addr.Map.find pc mutated_vars)
               with Not_found -> all_mut
@@ -300,7 +317,7 @@ let rewrite_mutable
   then free_pc, blocks, internal_and_external
   else
     match internal_and_external with
-    | [ Let (x, Closure (params, (pc, pc_args))) ] ->
+    | [ (Let (x, Closure (params, (pc, pc_args))), loc) ] ->
         let new_pc = free_pc in
         let free_pc = free_pc + 1 in
         let closure = Code.Var.fork x in
@@ -310,15 +327,15 @@ let rewrite_mutable
         rewrite_list := (mapping, pc) :: !rewrite_list;
         let new_block =
           { params = []
-          ; handler = None
-          ; body = [ Let (new_x, Closure (params, (pc, List.map pc_args ~f:mapping))) ]
-          ; branch = Return new_x
+          ; body =
+              [ Let (new_x, Closure (params, (pc, List.map pc_args ~f:mapping))), loc ]
+          ; branch = Return new_x, loc
           }
         in
         let blocks = Addr.Map.add new_pc new_block blocks in
         let body =
-          [ Let (closure, Closure (args, (new_pc, [])))
-          ; Let (x, Apply (closure, vars, true))
+          [ Let (closure, Closure (args, (new_pc, []))), noloc
+          ; Let (x, Apply { f = closure; args = vars; exact = true }), loc
           ]
         in
         free_pc, blocks, body
@@ -331,12 +348,12 @@ let rewrite_mutable
         let args = List.map vars ~f:Code.Var.fork in
         let pcs =
           List.map internal_and_external ~f:(function
-              | Let (_, Closure (_, (pc, _))) -> pc
+              | Let (_, Closure (_, (pc, _))), _ -> pc
               | _ -> assert false)
         in
         let old_xs =
           List.map closures_extern ~f:(function
-              | Let (x, Closure _) -> x
+              | Let (x, Closure _), _ -> x
               | _ -> assert false)
         in
         let new_xs = List.map old_xs ~f:Code.Var.fork in
@@ -348,44 +365,62 @@ let rewrite_mutable
           let proj =
             List.map2 closures_extern new_xs ~f:(fun cl new_x ->
                 match cl with
-                | Let (_, Closure (params, (pc, pc_args))) ->
-                    Let (new_x, Closure (params, (pc, List.map pc_args ~f:mapping)))
+                | Let (_, Closure (params, (pc, pc_args))), loc ->
+                    Let (new_x, Closure (params, (pc, List.map pc_args ~f:mapping))), loc
                 | _ -> assert false)
           in
           { params = []
-          ; handler = None
           ; body =
               closures_intern
               @ proj
-              @ [ Let (b, Block (0, Array.of_list new_xs, NotArray)) ]
-          ; branch = Return b
+              @ [ Let (b, Block (0, Array.of_list new_xs, NotArray)), noloc ]
+          ; branch = Return b, noloc
           }
         in
         let blocks = Addr.Map.add new_pc new_block blocks in
         let body =
-          [ Let (closure, Closure (args, (new_pc, [])))
-          ; Let (closure', Apply (closure, vars, true))
+          [ Let (closure, Closure (args, (new_pc, []))), noloc
+          ; Let (closure', Apply { f = closure; args = vars; exact = true }), noloc
           ]
           @ List.mapi closures_extern ~f:(fun i x ->
                 match x with
-                | Let (x, Closure _) -> Let (x, Field (closure', i))
+                | Let (x, Closure _), loc -> Let (x, Field (closure', i)), loc
                 | _ -> assert false)
         in
         free_pc, blocks, body
 
+let sort_components closures_map components =
+  let pos_of_var x = (Var.Map.find x closures_map).pos in
+  let pos_of_comp : _ -> int = function
+    | SCC.No_loop x -> pos_of_var x
+    | SCC.Has_loop all ->
+        List.fold_left all ~init:max_int ~f:(fun acc x -> min (pos_of_var x) acc)
+  in
+
+  List.sort components ~cmp:(fun x y -> compare (pos_of_comp x) (pos_of_comp y))
+  |> List.map ~f:(function
+         | SCC.No_loop _ as x -> x
+         | SCC.Has_loop l ->
+             SCC.Has_loop
+               (List.sort l ~cmp:(fun a b -> compare (pos_of_var a) (pos_of_var b))))
+
 let rec rewrite_closures mutated_vars rewrite_list free_pc blocks body : int * _ * _ list
     =
   match body with
-  | Let (_, Closure _) :: _ ->
-      let closures, rem = collect_closures blocks body in
+  | (Let (_, Closure _), _) :: _ ->
+      let closures, rem = collect_closures blocks mutated_vars body 0 in
       let closures_map =
         List.fold_left closures ~init:Var.Map.empty ~f:(fun closures_map x ->
             Var.Map.add x.f_name x closures_map)
       in
-      let components = group_closures ~tc_only:false closures_map in
+      let components =
+        group_closures ~tc_only:false closures_map
+        |> Array.to_list
+        |> sort_components closures_map
+      in
       let free_pc, blocks, closures =
         List.fold_left
-          (Array.to_list components)
+          components
           ~init:(free_pc, blocks, [])
           ~f:(fun (free_pc, blocks, acc) component ->
             let free_pc, blocks, closures =
@@ -399,6 +434,7 @@ let rec rewrite_closures mutated_vars rewrite_list free_pc blocks body : int * _
                          (fun v _ -> List.exists all ~f:(Var.equal v))
                          closures_map)
                     |> Array.to_list
+                    |> sort_components closures_map
               in
               List.fold_left
                 ~init:(free_pc, blocks, { int = []; ext = [] })
@@ -422,7 +458,7 @@ let rec rewrite_closures mutated_vars rewrite_list free_pc blocks body : int * _
       let free_pc, blocks, rem =
         rewrite_closures mutated_vars rewrite_list free_pc blocks rem
       in
-      free_pc, blocks, List.flatten closures @ rem
+      free_pc, blocks, List.flatten (List.rev closures) @ rem
   | i :: rem ->
       let free_pc, blocks, rem =
         rewrite_closures mutated_vars rewrite_list free_pc blocks rem
@@ -454,3 +490,9 @@ let f p : Code.program =
   in
   Code.invariant p;
   p
+
+let f p =
+  let t = Timer.make () in
+  let p' = f p in
+  if Debug.find "times" () then Format.eprintf "  generate closures: %a@." Timer.print t;
+  p'
