@@ -169,6 +169,10 @@ type t =
   | RERAISE
   | RAISE_NOTRACE
   | GETSTRINGCHAR
+  | PERFORM
+  | RESUME
+  | RESUMETERM
+  | REPERFORMTERM
   | FIRST_UNIMPLEMENTED_OP
 
 type kind =
@@ -195,10 +199,11 @@ type desc =
   }
 
 let ops =
-  let if_v407 =
+  let if_v500 =
     match Ocaml_version.v with
-    | `V4_04 | `V4_06 -> fun _ -> K_will_not_happen
-    | `V4_07 | `V4_08 | `V4_09 | `V4_10 | `V4_11 | `V4_12 | `V4_13 -> fun k -> k
+    | `V4_08 | `V4_09 | `V4_10 | `V4_11 | `V4_12 | `V4_13 | `V4_14 ->
+        fun _ -> K_will_not_happen
+    | `V5_00 | `V5_01 | `V5_02 -> fun k -> k
   in
   let instrs =
     [| ACC0, KNullary, "ACC0"
@@ -349,7 +354,11 @@ let ops =
      ; BREAK, K_will_not_happen, "BREAK"
      ; RERAISE, KStop 0, "RERAISE"
      ; RAISE_NOTRACE, KStop 0, "RAISE_NOTRACE"
-     ; GETSTRINGCHAR, if_v407 KNullary, "GETSTRINGCHAR"
+     ; GETSTRINGCHAR, KNullary, "GETSTRINGCHAR"
+     ; PERFORM, if_v500 KNullaryCall, "PERFORM"
+     ; RESUME, if_v500 KNullaryCall, "RESUME"
+     ; RESUMETERM, if_v500 (KStop 1), "RESUMETERM"
+     ; REPERFORMTERM, if_v500 (KStop 1), "REPERFORMTERM"
      ; FIRST_UNIMPLEMENTED_OP, K_will_not_happen, "FIRST_UNIMPLEMENTED_OP"
     |]
   in
