@@ -19,4 +19,8 @@
  *)
 
 val program :
-  Pretty_print.t -> ?source_map:string option * Source_map.t -> Javascript.program -> unit
+     ?accept_unnamed_var:bool
+  -> Pretty_print.t
+  -> ?source_map:Source_map.t
+  -> Javascript.program
+  -> Source_map.t option
