@@ -24,20 +24,24 @@ module Fragment : sig
   type t
 
   val provides : t -> string list
+
+  val parse_file : string -> t list
+
+  val parse_string : string -> t list
+
+  val parse_builtin : Builtins.File.t -> t list
+
+  val pack : t -> t
 end
 
 val reset : unit -> unit
 
-val parse_file : string -> Fragment.t list
-
-val parse_string : string -> Fragment.t list
-
-val parse_builtin : Builtins.File.t -> Fragment.t list
-
 val load_files : target_env:Target_env.t -> string list -> unit
 
 val load_fragments : target_env:Target_env.t -> filename:string -> Fragment.t list -> unit
 
+val check_deps : unit -> unit
+
 type state
 
 type always_required =
