@@ -21,23 +21,53 @@ val constant_of_const : Lambda.structured_constant -> Code.constant
 val find_loc_in_summary : Ident.t -> Env.summary -> Location.t option
 
 module Symtable : sig
+  module Global : sig
+    type t =
+      | Glob_compunit of string
+      | Glob_predef of string
+
+    val name : t -> string
+
+    val of_ident : Ident.t -> t option
+  end
+
   module GlobalMap : sig
     type t
 
-    val filter_global_map : (Ident.t -> bool) -> t -> t
+    val empty : t
 
-    val find : Ident.t -> t -> int
+    val filter : (Global.t -> bool) -> t -> t
 
-    val iter : (Ident.t -> int -> unit) -> t -> unit
+    val find : Global.t -> t -> int
 
-    val fold : (Ident.t -> int -> 'a -> 'a) -> t -> 'a -> 'a
+    val iter : f:(Global.t -> int -> unit) -> t -> unit
+
+    val fold : (Global.t -> int -> 'a -> 'a) -> t -> 'a -> 'a
+
+    val enter : t ref -> Global.t -> int
   end
 
   val reloc_ident : string -> int
+
+  val current_state : unit -> GlobalMap.t
 end
 
 module Ident : sig
   type 'a tbl = 'a Ident.tbl
 
-  val table_contents : int -> int Ident.tbl -> (int * string * Ident.t) list
+  val table_contents : 'a Ident.tbl -> ('a * Ident.t) list
+end
+
+module Cmo_format : sig
+  type t = Cmo_format.compilation_unit
+
+  val name : t -> string
+
+  val requires : t -> string list
+
+  val primitives : t -> string list
+
+  val force_link : t -> bool
+
+  val imports : t -> (string * string option) list
 end
