@@ -20,12 +20,12 @@
 open Stdlib
 
 val f :
-     prim:[ `caml_create_file | `caml_create_file_extern ]
+     prim:[ `create_file | `create_file_extern ]
   -> cmis:StringSet.t
   -> files:string list
   -> paths:string list
-  -> Code.instr list
+  -> (Code.instr * Code.loc) list
 
-val embed_file : name:string -> filename:string -> Code.instr
+val embed_file : name:string -> filename:string -> Code.instr * Code.loc
 
-val init : unit -> Code.instr
+val init : unit -> Code.instr * Code.loc
