@@ -26,15 +26,15 @@ val expr : (Var.t -> Var.t) -> expr -> expr
 
 val instr : (Var.t -> Var.t) -> instr -> instr
 
-val instrs : (Var.t -> Var.t) -> instr list -> instr list
+val instrs : (Var.t -> Var.t) -> (instr * loc) list -> (instr * loc) list
 
 val block : (Var.t -> Var.t) -> block -> block
 
-val last : (Var.t -> Var.t) -> last -> last
+val last : (Var.t -> Var.t) -> last * loc -> last * loc
 
 val cont : (Var.t -> Var.t) -> int -> program -> program
 
-val from_array : Var.t option array -> Var.t -> Var.t
+val from_array : Var.t array -> Var.t -> Var.t
 
 val build_mapping : Var.t list -> Var.t list -> Var.t Var.Map.t
 
