@@ -1,4 +1,6 @@
 let () =
+  (* Make sure Printexc is linked *)
+  let _ = Printexc.to_string Not_found in
   match Array.to_list Sys.argv with
   | _ :: "unregister" :: _ ->
       let null = Array.unsafe_get [| 1 |] 1 in
