@@ -112,8 +112,12 @@ let%expect_test _ =
   let data =
     "\132\149\166\190\000\000\000\012\000\000\000\001\000\000\000\004\000\000\000\003\018_j\000\000\000\000\000\000\000\000\001"
   in
-  let v = Marshal.from_string data 0 in
-  let () = assert (1L = v) in
+  (if String.split_on_char '.' Sys.ocaml_version |> List.hd |> int_of_string >= 5
+   then ()
+   else
+     let v = Marshal.from_string data 0 in
+     let () = assert (1L = v) in
+     ());
   [%expect {||}]
 
 let%expect_test _ =
@@ -129,3 +133,36 @@ let%expect_test _ =
   Printf.printf "%S" (Marshal.to_string ba []);
   [%expect
     {| "\132\149\166\190\000\000\000'\000\000\000\001\000\000\000\007\000\000\000\007\024_bigarr02\000\000\000\000\020\000\000\000\000\000\000\000(\000\000\000\001\000\000\000\005\000\003\000\003\000\001\000\002" |}]
+
+let%expect_test _ =
+  let data =
+    "\132\149\166\189\r\022\206\021\001\147F\137d(\181/\253\000Xm\000\0000\n\
+     \000\000'\016c\001\000\012\135\007E"
+  in
+  let ocaml_5_1 =
+    match String.split_on_char '.' Sys.ocaml_version with
+    | major :: minor :: _ ->
+        let major = int_of_string major and minor = int_of_string minor in
+        major > 5 || (major = 5 && minor >= 1)
+    | _ -> assert false
+  in
+  let s =
+    (* This would only work on OCaml 5.1.0 if we were not linking
+       against compiler-libs *)
+    if ocaml_5_1 && not (Sys.win32 || Sys.cygwin)
+    then Marshal.from_string data 0
+    else String.make 10000 'c'
+  in
+  Printf.printf "%s ... (%d)\n" (String.sub s 0 20) (String.length s);
+  [%expect {| cccccccccccccccccccc ... (10000) |}]
+
+let%expect_test "test sharing of string" =
+  let s = "AString" in
+  let p = s, s in
+  let obj = [ p; p ] in
+  Printf.printf "%S" (Marshal.to_string s []);
+  [%expect
+    {| "\132\149\166\190\000\000\000\b\000\000\000\001\000\000\000\003\000\000\000\002'AString" |}];
+  Printf.printf "%S" (Marshal.to_string obj []);
+  [%expect
+    {| "\132\149\166\190\000\000\000\016\000\000\000\004\000\000\000\012\000\000\000\011\160\160'AString\004\001\160\004\003@" |}]
