@@ -20,15 +20,19 @@
 //Provides: fs_node_supported
 function fs_node_supported () {
   return (
-    typeof joo_global_object.process !== 'undefined'
-      && typeof joo_global_object.process.versions !== 'undefined'
-      && typeof joo_global_object.process.versions.node !== 'undefined'
-      && joo_global_object.process.platform !== "browser")
+    typeof globalThis.process !== 'undefined'
+      && typeof globalThis.process.versions !== 'undefined'
+      && typeof globalThis.process.versions.node !== 'undefined')
+}
+//Provides: fs_node_supported
+//If: browser
+function fs_node_supported () {
+  return false
 }
 
 
 //Provides: MlNodeDevice
-//Requires: MlNodeFile, caml_raise_sys_error, caml_raise_with_args
+//Requires: MlNodeFd, caml_raise_sys_error, caml_raise_with_args
 //Requires: make_unix_err_args, caml_named_value, caml_string_of_jsstring
 function MlNodeDevice(root) {
   this.fs = require('fs');
@@ -44,6 +48,13 @@ MlNodeDevice.prototype.exists = function(name) {
     return 0;
   }
 }
+MlNodeDevice.prototype.isFile = function(name) {
+  try {
+    return this.fs.statSync(this.nm(name)).isFile()?1:0;
+  } catch (err) {
+    caml_raise_sys_error(err.toString());
+  }
+}
 MlNodeDevice.prototype.mkdir = function(name, mode, raise_unix) {
   try {
     this.fs.mkdirSync(this.nm(name),{mode:mode});
@@ -103,7 +114,9 @@ MlNodeDevice.prototype.open = function(name, f, raise_unix) {
   }
   try {
     var fd = this.fs.openSync(this.nm(name), res);
-    return new MlNodeFile(fd);
+    var isCharacterDevice = this.fs.lstatSync(this.nm(name)).isCharacterDevice();
+    f.isCharacterDevice = isCharacterDevice;
+    return new MlNodeFd(fd, f);
   } catch (err) {
     this.raise_nodejs_error(err, raise_unix);
   }
@@ -148,6 +161,13 @@ MlNodeDevice.prototype.readlink = function(name, raise_unix) {
     this.raise_nodejs_error(err, raise_unix);
   }
 }
+MlNodeDevice.prototype.opendir = function(name, raise_unix) {
+  try {
+    return this.fs.opendirSync(this.nm(name));
+  } catch (err) {
+    this.raise_nodejs_error(err, raise_unix);
+  }
+}
 MlNodeDevice.prototype.raise_nodejs_error = function(err, raise_unix) {
   var unix_error = caml_named_value("Unix.Unix_error");
   if (raise_unix && unix_error) {
@@ -219,71 +239,88 @@ MlNodeDevice.prototype.stats_from_js = function(js_stats) {
 
 MlNodeDevice.prototype.constructor = MlNodeDevice
 
-//Provides: MlNodeFile
-//Requires: MlFile, caml_array_of_string, caml_array_of_bytes, caml_bytes_set, caml_raise_sys_error
-function MlNodeFile(fd){
+//Provides: MlNodeDevice
+//If: browser
+function MlNodeDevice() {
+}
+
+//Provides: MlNodeFd
+//Requires: MlFile, caml_uint8_array_of_string, caml_uint8_array_of_bytes, caml_bytes_set, caml_raise_sys_error
+function MlNodeFd(fd, flags){
   this.fs = require('fs');
   this.fd = fd;
+  this.flags = flags;
 }
-MlNodeFile.prototype = new MlFile ();
+MlNodeFd.prototype = new MlFile ();
+MlNodeFd.prototype.constructor = MlNodeFd;
 
-MlNodeFile.prototype.truncate = function(len){
+MlNodeFd.prototype.truncate = function(len){
   try {
-    this.fs.ftruncateSync(this.fd,len|0)
+    this.fs.ftruncateSync(this.fd,len|0);
   } catch (err) {
     caml_raise_sys_error(err.toString());
   }
 }
-MlNodeFile.prototype.length = function () {
+MlNodeFd.prototype.length = function () {
   try {
     return this.fs.fstatSync(this.fd).size;
   } catch (err) {
     caml_raise_sys_error(err.toString());
   }
 }
-MlNodeFile.prototype.write = function(offset,buf,buf_offset,len){
-  var a = caml_array_of_string(buf);
-  if(! (a instanceof joo_global_object.Uint8Array))
-    a = new joo_global_object.Uint8Array(a);
-  var buffer = joo_global_object.Buffer.from(a);
+MlNodeFd.prototype.write = function(offset,buf,buf_offset,len){
   try {
-    this.fs.writeSync(this.fd, buffer, buf_offset, len, offset);
+    if(this.flags.isCharacterDevice)
+      this.fs.writeSync(this.fd, buf, buf_offset, len);
+    else
+      this.fs.writeSync(this.fd, buf, buf_offset, len, offset);
   } catch (err) {
     caml_raise_sys_error(err.toString());
   }
   return 0;
 }
-MlNodeFile.prototype.read = function(offset,buf,buf_offset,len){
-  var a = caml_array_of_bytes(buf);
-  if(! (a instanceof joo_global_object.Uint8Array))
-    a = new joo_global_object.Uint8Array(a);
-  var buffer = joo_global_object.Buffer.from(a);
+MlNodeFd.prototype.read = function(offset,a,buf_offset,len){
   try {
-    this.fs.readSync(this.fd, buffer, buf_offset, len, offset);
+    if(this.flags.isCharacterDevice)
+      var read = this.fs.readSync(this.fd, a, buf_offset, len);
+    else
+      var read = this.fs.readSync(this.fd, a, buf_offset, len, offset);
+    return read;
   } catch (err) {
     caml_raise_sys_error(err.toString());
   }
-  for(var i = 0; i < len; i++){
-    caml_bytes_set(buf,buf_offset + i,buffer[buf_offset+i]);
-  }
-  return 0
 }
-MlNodeFile.prototype.read_one = function(offset){
-  var a = new joo_global_object.Uint8Array(1);
-  var buffer = joo_global_object.Buffer.from(a);
-  try {
-    this.fs.readSync(this.fd, buffer, 0, 1, offset);
-  } catch (err) {
-    caml_raise_sys_error(err.toString());
-  }
-  return buffer[0];
-}
-MlNodeFile.prototype.close = function(){
+MlNodeFd.prototype.close = function(){
   try {
     this.fs.closeSync(this.fd);
+    return 0
   } catch (err) {
     caml_raise_sys_error(err.toString());
   }
 }
 
-MlNodeFile.prototype.constructor = MlNodeFile;
+
+//Provides: MlNodeFd
+//If: browser
+function MlNodeFd(){
+}
+
+
+//Provides: caml_sys_open_for_node
+//Requires: MlNodeFd
+function caml_sys_open_for_node(fd, flags){
+  if(flags.name) {
+    try {
+      var fs = require("fs");
+      var fd2 = fs.openSync(flags.name, "rs");
+      return new MlNodeFd(fd2, flags);
+    } catch(e) {  }
+  }
+  return new MlNodeFd(fd, flags);
+}
+
+//Provides: caml_sys_open_for_node
+//If: browser
+function caml_sys_open_for_node(fd, flags){
+  return null;
+}
