@@ -67,9 +67,9 @@ function caml_int64_bits_of_float (x) {
 //Provides: caml_int32_bits_of_float const
 //Requires: jsoo_floor_log2
 function caml_int32_bits_of_float (x) {
-  var float32a = new joo_global_object.Float32Array(1);
+  var float32a = new Float32Array(1);
   float32a[0] = x;
-  var int32a = new joo_global_object.Int32Array(float32a.buffer);
+  var int32a = new Int32Array(float32a.buffer);
   return int32a[0] | 0;
 }
 
@@ -174,9 +174,9 @@ function caml_trunc_float(x){
 
 //Provides: caml_int32_float_of_bits const
 function caml_int32_float_of_bits (x) {
-  var int32a = new joo_global_object.Int32Array(1);
+  var int32a = new Int32Array(1);
   int32a[0] = x;
-  var float32a = new joo_global_object.Float32Array(int32a.buffer);
+  var float32a = new Float32Array(int32a.buffer);
   return float32a[0];
 }
 
@@ -288,7 +288,15 @@ function caml_tanh_float (x) { return Math.tanh(x); }
 //Provides: caml_atanh_float const
 function caml_atanh_float (x) { return Math.atanh(x); }
 //Provides: caml_round_float const
-function caml_round_float (x) { return Math.round(x); }
+function caml_round_float (x) {
+  if (x >= 0) {
+    var y = Math.floor(x);
+    return (x - y >= 0.5)?(y + 1):y
+  } else {
+    var y = Math.ceil(x);
+    return (y - x >= 0.5)?(y - 1):y
+  }
+}
 //Provides: caml_cbrt_float const
 function caml_cbrt_float (x) { return Math.cbrt(x); }
 
@@ -498,12 +506,12 @@ function caml_float_of_string(s) {
   s = s.replace(/_/g,"");
   res = +s;
   if (((s.length > 0) && (res === res)) || /^[+-]?nan$/i.test(s)) return res;
-  var m = /^ *([+-]?)0x([0-9a-f]+)\.?([0-9a-f]*)p([+-]?[0-9]+)/i.exec(s);
-  //          1        2             3           4
+  var m = /^ *([+-]?)0x([0-9a-f]+)\.?([0-9a-f]*)(p([+-]?[0-9]+))?/i.exec(s);
+  //          1        2             3           5
   if(m){
     var m3 = m[3].replace(/0+$/,'');
     var mantissa = parseInt(m[1] + m[2] + m3, 16);
-    var exponent = (m[4]|0) - 4*m3.length;
+    var exponent = (m[5]|0) - 4*m3.length;
     res = mantissa * Math.pow(2, exponent);
     return res;
   }
