@@ -20,11 +20,12 @@
 ///////////// Jslib
 
 //Provides: caml_js_pure_expr const
-function caml_js_pure_expr (f) { return f(); }
+//Requires: caml_callback
+function caml_js_pure_expr (f) { return caml_callback(f, [0]); }
 
-//Provides: caml_js_set (mutable, const, const)
+//Provides: caml_js_set (mutable, const, mutable)
 function caml_js_set(o,f,v) { o[f]=v;return 0}
-//Provides: caml_js_get mutable (const, const)
+//Provides: caml_js_get (mutable, const)
 function caml_js_get(o,f) { return o[f]; }
 //Provides: caml_js_delete (mutable, const)
 function caml_js_delete(o,f) { delete o[f]; return 0}
@@ -50,37 +51,65 @@ function caml_trampoline_return(f,args) {
   return {joo_tramp:f,joo_args:args};
 }
 
-//Provides: js_print_stdout (const)
-//Requires: caml_utf16_of_utf8
-function js_print_stdout(s) {
-  var s = caml_utf16_of_utf8(s);
-  var g = globalThis;
-  if (g.process && g.process.stdout && g.process.stdout.write) {
-    g.process.stdout.write(s)
-  } else {
-    // Do not output the last \n if present
-    // as console logging display a newline at the end
-    if(s.charCodeAt(s.length - 1) == 10)
-      s = s.substr(0,s.length - 1 );
-    var v = g.console;
-    v  && v.log && v.log(s);
-  }
+//Provides:caml_stack_depth
+//If: effects
+var caml_stack_depth = 0;
+
+//Provides:caml_stack_check_depth
+//If: effects
+//Requires:caml_stack_depth
+function caml_stack_check_depth() {
+    return --caml_stack_depth > 0;
 }
-//Provides: js_print_stderr (const)
-//Requires: caml_utf16_of_utf8
-function js_print_stderr(s) {
-  var s = caml_utf16_of_utf8(s);
-  var g = globalThis;
-  if (g.process && g.process.stdout && g.process.stdout.write) {
-    g.process.stderr.write(s)
-  } else {
-    // Do not output the last \n if present
-    // as console logging display a newline at the end
-    if(s.charCodeAt(s.length - 1) == 10)
-      s = s.substr(0,s.length - 1 );
-    var v = g.console;
-    v && v.error && v.error(s);
+
+//Provides: caml_callback
+//If: !effects
+//Requires:caml_call_gen
+var caml_callback = caml_call_gen;
+
+//Provides: caml_callback
+//If: effects
+//Requires:caml_stack_depth, caml_call_gen, caml_exn_stack, caml_fiber_stack, caml_wrap_exception, caml_resume_stack, caml_fresh_oo_id, caml_named_value, caml_raise_with_arg, caml_string_of_jsbytes
+//Requires: caml_raise_constant
+function caml_callback(f,args) {
+  function uncaught_effect_handler(eff,k,ms) {
+    // Resumes the continuation k by raising exception Unhandled.
+    caml_resume_stack(k[1],ms);
+    var exn = caml_named_value("Effect.Unhandled");
+    if(exn) caml_raise_with_arg(exn, eff);
+    else {
+      exn = [248,caml_string_of_jsbytes("Effect.Unhandled"), caml_fresh_oo_id(0)];
+      caml_raise_constant(exn);
+    }
   }
+  var saved_stack_depth = caml_stack_depth;
+  var saved_exn_stack = caml_exn_stack;
+  var saved_fiber_stack = caml_fiber_stack;
+  try {
+    caml_exn_stack = 0;
+    caml_fiber_stack =
+      {h:[0, 0, 0, uncaught_effect_handler], r:{k:0, x:0, e:0}};
+    var res = {joo_tramp: f,
+               joo_args: args.concat(function (x){return x;})};
+    do {
+      caml_stack_depth = 40;
+      try {
+        res = caml_call_gen(res.joo_tramp, res.joo_args);
+      } catch (e) {
+        /* Handle exception coming from JavaScript or from the runtime. */
+        if (!caml_exn_stack) throw e;
+        var handler = caml_exn_stack[1];
+        caml_exn_stack = caml_exn_stack[2];
+        res = {joo_tramp: handler,
+               joo_args: [caml_wrap_exception(e)]};
+      }
+    } while(res && res.joo_args)
+  } finally {
+    caml_stack_depth = saved_stack_depth;
+    caml_exn_stack = saved_exn_stack;
+    caml_fiber_stack = saved_fiber_stack;
+  }
+  return res;
 }
 
 //Provides: caml_is_js
@@ -88,28 +117,55 @@ function caml_is_js() {
   return 1;
 }
 
-//Provides: caml_wrap_exception const (const)
+//Provides: caml_jsoo_flags_use_js_string
+function caml_jsoo_flags_use_js_string(unit){
+  return FLAG("use-js-string")
+}
+
+//Provides: caml_jsoo_flags_effects
+function caml_jsoo_flags_effects(unit){
+  return FLAG("effects")
+}
+
+//Provides: caml_wrap_exception const (mutable)
 //Requires: caml_global_data,caml_string_of_jsstring,caml_named_value
-//Requires: caml_return_exn_constant
 function caml_wrap_exception(e) {
-  if(e instanceof Array) return e;
-  //Stack_overflow: chrome, safari
-  if(globalThis.RangeError
-     && e instanceof globalThis.RangeError
-     && e.message
-     && e.message.match(/maximum call stack/i))
-    return caml_return_exn_constant(caml_global_data.Stack_overflow);
-  //Stack_overflow: firefox
-  if(globalThis.InternalError
-     && e instanceof globalThis.InternalError
-     && e.message
-     && e.message.match(/too much recursion/i))
-    return caml_return_exn_constant(caml_global_data.Stack_overflow);
-  //Wrap Error in Js.Error exception
-  if(e instanceof globalThis.Error && caml_named_value("jsError"))
-    return [0,caml_named_value("jsError"),e];
-  //fallback: wrapped in Failure
-  return [0,caml_global_data.Failure,caml_string_of_jsstring (String(e))];
+  if (FLAG("excwrap")) {
+    if(e instanceof Array) return e;
+    var exn;
+    //Stack_overflow: chrome, safari
+    if(globalThis.RangeError
+       && e instanceof globalThis.RangeError
+       && e.message
+       && e.message.match(/maximum call stack/i))
+      exn = caml_global_data.Stack_overflow;
+    //Stack_overflow: firefox
+    else if(globalThis.InternalError
+       && e instanceof globalThis.InternalError
+       && e.message
+       && e.message.match(/too much recursion/i))
+      exn = caml_global_data.Stack_overflow;
+    //Wrap Error in Js.Error exception
+    else if(e instanceof globalThis.Error && caml_named_value("jsError"))
+      exn = [0,caml_named_value("jsError"),e];
+    else
+      //fallback: wrapped in Failure
+      exn = [0,caml_global_data.Failure,caml_string_of_jsstring (String(e))];
+    // We already have an error at hand, let's use it.
+    if (e instanceof globalThis.Error)
+      exn.js_error = e;
+    return exn;
+  } else
+    return e;
+}
+
+//Provides: caml_maybe_attach_backtrace
+//Requires: caml_exn_with_js_backtrace
+//Requires: caml_record_backtrace_flag
+function caml_maybe_attach_backtrace(exn, force) {
+  if(caml_record_backtrace_flag)
+    return caml_exn_with_js_backtrace(exn, force);
+  else return exn
 }
 
 // Experimental
@@ -135,9 +191,14 @@ function caml_js_from_bool(x) { return !!x; }
 //Provides: caml_js_to_bool const (const)
 function caml_js_to_bool(x) { return +x; }
 //Provides: caml_js_from_float const (const)
+//Alias: caml_js_from_int32
+//Alias: caml_js_from_nativeint
 function caml_js_from_float(x) { return x; }
 //Provides: caml_js_to_float const (const)
 function caml_js_to_float(x) { return x; }
+//Provides: caml_js_to_int32 const (const)
+//Alias: caml_js_to_nativeint
+function caml_js_to_int32(x) { return x|0; }
 
 //Provides: caml_js_from_array mutable (shallow)
 function caml_js_from_array(a) {
@@ -152,7 +213,7 @@ function caml_js_to_array(a) {
   return b;
 }
 
-//Provides: caml_list_of_js_array const (const)
+//Provides: caml_list_of_js_array const (mutable)
 function caml_list_of_js_array(a){
   var l = 0;
   for(var i=a.length - 1; i>=0; i--){
@@ -162,7 +223,7 @@ function caml_list_of_js_array(a){
   return l
 }
 
-//Provides: caml_list_to_js_array const (const)
+//Provides: caml_list_to_js_array const (mutable)
 function caml_list_to_js_array(l){
   var a = [];
   for(; l !== 0; l = l[2]) {
@@ -171,15 +232,14 @@ function caml_list_to_js_array(l){
   return a;
 }
 
-//Provides: caml_js_var mutable (const)
-//Requires: js_print_stderr
+//Provides: caml_js_var mutable
 //Requires: caml_jsstring_of_string
 function caml_js_var(x) {
   var x = caml_jsstring_of_string(x);
   //Checks that x has the form ident[.ident]*
   if(!x.match(/^[a-zA-Z_$][a-zA-Z_$0-9]*(\.[a-zA-Z_$][a-zA-Z_$0-9]*)*$/)){
-    js_print_stderr("caml_js_var: \"" + x + "\" is not a valid JavaScript variable. continuing ..");
-    //globalThis.console.error("Js.Unsafe.eval_string")
+    console.error("caml_js_var: \"" + x + "\" is not a valid JavaScript variable. continuing ..");
+    //console.error("Js.Unsafe.eval_string")
   }
   return eval(x);
 }
@@ -242,103 +302,129 @@ function caml_ojs_new_arr(c, a) {
   return new F;
 }
 //Provides: caml_js_wrap_callback const (const)
-//Requires: caml_call_gen
+//Requires: caml_callback
 function caml_js_wrap_callback(f) {
   return function () {
     var len = arguments.length;
     if(len > 0){
       var args = new Array(len);
       for (var i = 0; i < len; i++) args[i] = arguments[i];
-      return caml_call_gen(f, args);
     } else {
-      return caml_call_gen(f, [undefined]);
+      args = [undefined];
     }
+    var res = caml_callback(f, args);
+    return (res instanceof Function)?caml_js_wrap_callback(res):res;
   }
 }
 
 //Provides: caml_js_wrap_callback_arguments
-//Requires: caml_call_gen
+//Requires: caml_callback
 function caml_js_wrap_callback_arguments(f) {
   return function() {
     var len = arguments.length;
     var args = new Array(len);
     for (var i = 0; i < len; i++) args[i] = arguments[i];
-    return caml_call_gen(f, [args]);
+    return caml_callback(f, [args]);
   }
 }
 //Provides: caml_js_wrap_callback_strict const
-//Requires: caml_call_gen
+//Requires: caml_callback
 function caml_js_wrap_callback_strict(arity, f) {
   return function () {
     var n = arguments.length;
-    if(n == arity && f.length == arity) return f.apply(null, arguments);
     var args = new Array(arity);
     var len = Math.min(arguments.length, arity)
     for (var i = 0; i < len; i++) args[i] = arguments[i];
-    return caml_call_gen(f, args);
+    return caml_callback(f, args);
   };
 }
+//Provides: caml_js_wrap_callback_unsafe const (const)
+//Requires: caml_callback, caml_js_function_arity
+function caml_js_wrap_callback_unsafe(f) {
+  return function () {
+    var len = caml_js_function_arity(f);
+    var args = new Array(len);
+    for (var i = 0; i < len; i++) args[i] = arguments[i];
+    return caml_callback(f, args); }
+}
 //Provides: caml_js_wrap_meth_callback const (const)
-//Requires: caml_call_gen
+//Requires: caml_callback, caml_js_wrap_callback
 function caml_js_wrap_meth_callback(f) {
   return function () {
     var len = arguments.length;
     var args = new Array(len + 1);
     args[0] = this;
     for (var i = 0; i < len; i++) args[i+1] = arguments[i];
-    return caml_call_gen(f,args);
+    var res = caml_callback(f,args);
+    return (res instanceof Function)?caml_js_wrap_callback(res):res;
   }
 }
 //Provides: caml_js_wrap_meth_callback_arguments const (const)
-//Requires: caml_call_gen
+//Requires: caml_callback
 function caml_js_wrap_meth_callback_arguments(f) {
   return function () {
     var len = arguments.length;
     var args = new Array(len);
     for (var i = 0; i < len; i++) args[i] = arguments[i];
-    return caml_call_gen(f,[this,args]);
+    return caml_callback(f,[this,args]);
   }
 }
 //Provides: caml_js_wrap_meth_callback_strict const
-//Requires: caml_call_gen
+//Requires: caml_callback
 function caml_js_wrap_meth_callback_strict(arity, f) {
   return function () {
     var args = new Array(arity + 1);
     var len = Math.min(arguments.length, arity)
     args[0] = this;
     for (var i = 0; i < len; i++) args[i+1] = arguments[i];
-    return caml_call_gen(f, args);
+    return caml_callback(f, args);
   };
 }
 //Provides: caml_js_wrap_meth_callback_unsafe const (const)
-//Requires: caml_call_gen
+//Requires: caml_callback, caml_js_function_arity
 function caml_js_wrap_meth_callback_unsafe(f) {
   return function () {
-    var len = arguments.length;
+    var len = caml_js_function_arity(f) - 1;
     var args = new Array(len + 1);
     args[0] = this;
     for (var i = 0; i < len; i++) args[i+1] = arguments[i];
-    return f.apply(null, args); }
+    return caml_callback(f, args); }
 }
+
+//Provides: caml_js_function_arity
+//If: !effects
+function caml_js_function_arity(f) {
+  return (f.l >= 0)?f.l:(f.l = f.length)
+}
+
+//Provides: caml_js_function_arity
+//If: effects
+function caml_js_function_arity(f) {
+  // Functions have an additional continuation parameter. This should
+  // not be visible when calling them from JavaScript
+  return ((f.l >= 0)?f.l:(f.l = f.length)) - 1
+}
+
 //Provides: caml_js_equals mutable (const, const)
 function caml_js_equals (x, y) { return +(x == y); }
 
+//Provides: caml_js_strict_equals mutable (const, const)
+function caml_js_strict_equals (x, y) { return +(x === y); }
+
 //Provides: caml_js_eval_string (const)
 //Requires: caml_jsstring_of_string
 function caml_js_eval_string (s) {return eval(caml_jsstring_of_string(s));}
 
 //Provides: caml_js_expr (const)
-//Requires: js_print_stderr
 //Requires: caml_jsstring_of_string
 function caml_js_expr(s) {
-  js_print_stderr("caml_js_expr: fallback to runtime evaluation\n");
+  console.error("caml_js_expr: fallback to runtime evaluation\n");
   return eval(caml_jsstring_of_string(s));}
 
 //Provides: caml_pure_js_expr const (const)
-//Requires: js_print_stderr
 //Requires: caml_jsstring_of_string
 function caml_pure_js_expr (s){
-  js_print_stderr("caml_pure_js_expr: fallback to runtime evaluation\n");
+  console.error("caml_pure_js_expr: fallback to runtime evaluation\n");
   return eval(caml_jsstring_of_string(s));}
 
 //Provides: caml_js_object (object_literal)
