@@ -452,11 +452,7 @@ function caml_convert_string_to_bytes (s) {
 //Provides: caml_convert_bytes_to_array
 function caml_convert_bytes_to_array (s) {
   /* Assumes not ARRAY */
-  if(joo_global_object.Uint8Array) {
-    var a = new joo_global_object.Uint8Array(s.l);
-  } else {
-    var a = new Array(s.l);
-  }
+  var a = new Uint8Array(s.l);
   var b = s.c, l = b.length, i = 0;
   for (; i < l; i++) a[i] = b.charCodeAt(i);
   for (l = s.l; i < l; i++) a[i] = 0;
@@ -465,17 +461,17 @@ function caml_convert_bytes_to_array (s) {
   return a;
 }
 
-//Provides: caml_array_of_bytes mutable
+//Provides: caml_uint8_array_of_bytes mutable
 //Requires: caml_convert_bytes_to_array
-function caml_array_of_bytes (s) {
+function caml_uint8_array_of_bytes (s) {
   if (s.t != 4 /* ARRAY */) caml_convert_bytes_to_array(s);
   return s.c;
 }
 
-//Provides: caml_array_of_string mutable
+//Provides: caml_uint8_array_of_string mutable
 //Requires: caml_convert_bytes_to_array
 //Requires: caml_ml_string_length, caml_string_unsafe_get
-function caml_array_of_string (s) {
+function caml_uint8_array_of_string (s) {
   var l = caml_ml_string_length(s);
   var a = new Array(l);
   var i = 0;
@@ -514,6 +510,9 @@ function caml_string_of_array (a) {
 //Provides: caml_bytes_of_array
 //Requires: MlBytes
 function caml_bytes_of_array (a) {
+  if(! (a instanceof Uint8Array)) {
+    a = new Uint8Array(a);
+  }
   return new MlBytes(4,a,a.length);
 }
 
@@ -540,8 +539,8 @@ function caml_bytes_equal(s1, s2) {
 function caml_string_notequal(s1, s2) { return 1-caml_string_equal(s1, s2); }
 
 //Provides: caml_bytes_notequal mutable (const, const)
-//Requires: caml_string_equal
-function caml_bytes_notequal(s1, s2) { return 1-caml_string_equal(s1, s2); }
+//Requires: caml_bytes_equal
+function caml_bytes_notequal(s1, s2) { return 1-caml_bytes_equal(s1, s2); }
 
 //Provides: caml_bytes_lessequal mutable
 //Requires: caml_convert_string_to_bytes
@@ -584,6 +583,7 @@ function caml_bytes_greaterthan(s1, s2) {
 
 //Provides: caml_fill_bytes
 //Requires: caml_str_repeat, caml_convert_bytes_to_array
+//Alias: caml_fill_string
 function caml_fill_bytes(s, i, l, c) {
   if (l > 0) {
     if (i == 0 && (l >= s.l || (s.t == 2 /* PARTIAL */ && l >= s.c.length))) {
@@ -602,10 +602,6 @@ function caml_fill_bytes(s, i, l, c) {
   return 0;
 }
 
-//Provides: caml_fill_string
-//Requires: caml_fill_bytes
-var caml_fill_string = caml_fill_bytes
-
 //Provides: caml_blit_bytes
 //Requires: caml_subarray_to_jsbytes, caml_convert_bytes_to_array
 function caml_blit_bytes(s1, i1, s2, i2, len) {
@@ -806,7 +802,7 @@ function caml_string_of_jsbytes(s) { return caml_bytes_of_jsbytes(s); }
 //Requires: caml_convert_string_to_bytes
 //If: !js-string
 function caml_jsbytes_of_string(s) {
-  if ((s.t & 6) != 0 /* BYTES */) caml_convert_string_to_bytes(s);
+  (s.t & 6) && caml_convert_string_to_bytes(s);
   return s.c }
 
 //Provides: caml_jsstring_of_string mutable (const)
@@ -828,6 +824,19 @@ function caml_is_ml_bytes(s) {
   return (s instanceof MlBytes);
 }
 
+//Provides: caml_ml_bytes_content
+//Requires: MlBytes, caml_convert_string_to_bytes
+function caml_ml_bytes_content(s) {
+  switch (s.t & 6) {
+  default: /* PARTIAL */
+    caml_convert_string_to_bytes(s);
+  case 0: /* BYTES */
+    return s.c;
+  case 4:
+    return s.c
+  }
+}
+
 //Provides: caml_is_ml_string
 //Requires: jsoo_is_ascii
 //If: js-string
@@ -869,3 +878,12 @@ function caml_to_js_string(s) {
 function caml_js_to_string (s) {
   return caml_string_of_jsstring(s);
 }
+
+
+//Provides: caml_array_of_string
+//Requires: caml_uint8_array_of_string
+function caml_array_of_string(x) { return caml_uint8_array_of_string(x) }
+
+//Provides: caml_array_of_bytes
+//Requires: caml_uint8_array_of_bytes
+function caml_array_of_bytes(x) { return caml_uint8_array_of_bytes(x) }
