@@ -57,7 +57,7 @@ let rec scan_args acc = function
   | x :: xs -> scan_args (x :: acc) xs
   | [] -> List.rev acc
 
-let args =
+let () =
   let args = List.tl (Array.to_list Sys.argv) in
   let args = scan_args [] args in
   let runtime_files, args =
@@ -75,9 +75,10 @@ let args =
 
   List.iter builtin ~f:(fun t ->
       let filename = Js_of_ocaml_compiler.Builtins.File.name t in
-      let runtimes = Js_of_ocaml_compiler.Linker.parse_builtin t in
+      let runtimes = Js_of_ocaml_compiler.Linker.Fragment.parse_builtin t in
       Js_of_ocaml_compiler.Linker.load_fragments ~target_env:Isomorphic ~filename runtimes);
   Js_of_ocaml_compiler.Linker.load_files ~target_env:Isomorphic runtime_files;
+  Js_of_ocaml_compiler.Linker.check_deps ();
   let all = Jsoo_common.cmis args in
   let instr =
     List.map all ~f:(fun filename ->
@@ -93,7 +94,7 @@ let args =
   in
   let pfs_fmt = Js_of_ocaml_compiler.Pretty_print.to_out_channel oc in
   Js_of_ocaml_compiler.Config.Flag.enable "pretty";
-  Js_of_ocaml_compiler.Driver.f
+  Js_of_ocaml_compiler.Driver.f'
     pfs_fmt
-    (Js_of_ocaml_compiler.Parse_bytecode.Debug.create ~toplevel:false false)
+    (Js_of_ocaml_compiler.Parse_bytecode.Debug.create ~include_cmis:false false)
     program
